import sys
import typing


def add(radius: float = 1.0,
        type: typing.Union[int, str] = 'EMPTY',
        enter_editmode: bool = False,
        align: typing.Union[int, str] = 'WORLD',
        location: float = (0.0, 0.0, 0.0),
        rotation: float = (0.0, 0.0, 0.0)):
    '''Add an object to the scene 

    :param radius: Radius 
    :type radius: float
    :param type: Type 
    :type type: typing.Union[int, str]
    :param enter_editmode: Enter Editmode, Enter editmode when adding this object 
    :type enter_editmode: bool
    :param align: Align, The alignment of the new objectWORLD World, Align the new object to the world.VIEW View, Align the new object to the view.CURSOR 3D Cursor, Use the 3D cursor orientation for the new object. 
    :type align: typing.Union[int, str]
    :param location: Location, Location for the newly added object 
    :type location: float
    :param rotation: Rotation, Rotation for the newly added object 
    :type rotation: float
    '''

    pass


def add_named(linked: bool = False, name: str = ""):
    '''Add named object 

    :param linked: Linked, Duplicate object but not object data, linking to the original data 
    :type linked: bool
    :param name: Name, Object name to add 
    :type name: str
    '''

    pass


def align(bb_quality: bool = True,
          align_mode: typing.Union[int, str] = 'OPT_2',
          relative_to: typing.Union[int, str] = 'OPT_4',
          align_axis: typing.Set[typing.Union[int, str]] = {}):
    '''Align Objects 

    :param bb_quality: High Quality, Enables high quality calculation of the bounding box for perfect results on complex shape meshes with rotation/scale (Slow) 
    :type bb_quality: bool
    :param align_mode: Align Mode:, Side of object to use for alignment 
    :type align_mode: typing.Union[int, str]
    :param relative_to: Relative To:, Reference location to align toOPT_1 Scene Origin, Use the Scene Origin as the position for the selected objects to align to.OPT_2 3D Cursor, Use the 3D cursor as the position for the selected objects to align to.OPT_3 Selection, Use the selected objects as the position for the selected objects to align to.OPT_4 Active, Use the active object as the position for the selected objects to align to. 
    :type relative_to: typing.Union[int, str]
    :param align_axis: Align, Align to axis 
    :type align_axis: typing.Set[typing.Union[int, str]]
    '''

    pass


def anim_transforms_to_deltas():
    '''Convert object animation for normal transforms to delta transforms 

    '''

    pass


def armature_add(radius: float = 1.0,
                 enter_editmode: bool = False,
                 align: typing.Union[int, str] = 'WORLD',
                 location: float = (0.0, 0.0, 0.0),
                 rotation: float = (0.0, 0.0, 0.0)):
    '''Add an armature object to the scene 

    :param radius: Radius 
    :type radius: float
    :param enter_editmode: Enter Editmode, Enter editmode when adding this object 
    :type enter_editmode: bool
    :param align: Align, The alignment of the new objectWORLD World, Align the new object to the world.VIEW View, Align the new object to the view.CURSOR 3D Cursor, Use the 3D cursor orientation for the new object. 
    :type align: typing.Union[int, str]
    :param location: Location, Location for the newly added object 
    :type location: float
    :param rotation: Rotation, Rotation for the newly added object 
    :type rotation: float
    '''

    pass


def assign_property_defaults(process_data: bool = True,
                             process_bones: bool = True):
    '''Assign the current values of custom properties as their defaults, for use as part of the rest pose state in NLA track mixing 

    :param process_data: Process data properties 
    :type process_data: bool
    :param process_bones: Process bone properties 
    :type process_bones: bool
    '''

    pass


def bake(type: typing.Union[int, str] = 'COMBINED',
         pass_filter: typing.Set[typing.Union[int, str]] = {},
         filepath: str = "",
         width: int = 512,
         height: int = 512,
         margin: int = 16,
         use_selected_to_active: bool = False,
         cage_extrusion: float = 0.0,
         cage_object: str = "",
         normal_space: typing.Union[int, str] = 'TANGENT',
         normal_r: typing.Union[int, str] = 'POS_X',
         normal_g: typing.Union[int, str] = 'POS_Y',
         normal_b: typing.Union[int, str] = 'POS_Z',
         save_mode: typing.Union[int, str] = 'INTERNAL',
         use_clear: bool = False,
         use_cage: bool = False,
         use_split_materials: bool = False,
         use_automatic_name: bool = False,
         uv_layer: str = ""):
    '''Bake image textures of selected objects 

    :param type: Type, Type of pass to bake, some of them may not be supported by the current render engine 
    :type type: typing.Union[int, str]
    :param pass_filter: Pass Filter, Filter to combined, diffuse, glossy, transmission and subsurface passes 
    :type pass_filter: typing.Set[typing.Union[int, str]]
    :param filepath: File Path, Image filepath to use when saving externally 
    :type filepath: str
    :param width: Width, Horizontal dimension of the baking map (external only) 
    :type width: int
    :param height: Height, Vertical dimension of the baking map (external only) 
    :type height: int
    :param margin: Margin, Extends the baked result as a post process filter 
    :type margin: int
    :param use_selected_to_active: Selected to Active, Bake shading on the surface of selected objects to the active object 
    :type use_selected_to_active: bool
    :param cage_extrusion: Cage Extrusion, Distance to use for the inward ray cast when using selected to active 
    :type cage_extrusion: float
    :param cage_object: Cage Object, Object to use as cage, instead of calculating the cage from the active object with cage extrusion 
    :type cage_object: str
    :param normal_space: Normal Space, Choose normal space for bakingOBJECT Object, Bake the normals in object space.TANGENT Tangent, Bake the normals in tangent space. 
    :type normal_space: typing.Union[int, str]
    :param normal_r: R, Axis to bake in red channel 
    :type normal_r: typing.Union[int, str]
    :param normal_g: G, Axis to bake in green channel 
    :type normal_g: typing.Union[int, str]
    :param normal_b: B, Axis to bake in blue channel 
    :type normal_b: typing.Union[int, str]
    :param save_mode: Save Mode, Choose how to save the baking mapINTERNAL Internal, Save the baking map in an internal image data-block.EXTERNAL External, Save the baking map in an external file. 
    :type save_mode: typing.Union[int, str]
    :param use_clear: Clear, Clear Images before baking (only for internal saving) 
    :type use_clear: bool
    :param use_cage: Cage, Cast rays to active object from a cage 
    :type use_cage: bool
    :param use_split_materials: Split Materials, Split baked maps per material, using material name in output file (external only) 
    :type use_split_materials: bool
    :param use_automatic_name: Automatic Name, Automatically name the output file with the pass type 
    :type use_automatic_name: bool
    :param uv_layer: UV Layer, UV layer to override active 
    :type uv_layer: str
    '''

    pass


def bake_image():
    '''Bake image textures of selected objects 

    '''

    pass


def camera_add(enter_editmode: bool = False,
               align: typing.Union[int, str] = 'WORLD',
               location: float = (0.0, 0.0, 0.0),
               rotation: float = (0.0, 0.0, 0.0)):
    '''Add a camera object to the scene 

    :param enter_editmode: Enter Editmode, Enter editmode when adding this object 
    :type enter_editmode: bool
    :param align: Align, The alignment of the new objectWORLD World, Align the new object to the world.VIEW View, Align the new object to the view.CURSOR 3D Cursor, Use the 3D cursor orientation for the new object. 
    :type align: typing.Union[int, str]
    :param location: Location, Location for the newly added object 
    :type location: float
    :param rotation: Rotation, Rotation for the newly added object 
    :type rotation: float
    '''

    pass


def collection_add():
    '''Add an object to a new collection 

    '''

    pass


def collection_instance_add(name: str = "Collection",
                            collection: typing.Union[int, str] = '',
                            align: typing.Union[int, str] = 'WORLD',
                            location: float = (0.0, 0.0, 0.0),
                            rotation: float = (0.0, 0.0, 0.0)):
    '''Add a collection instance 

    :param name: Name, Collection name to add 
    :type name: str
    :param collection: Collection 
    :type collection: typing.Union[int, str]
    :param align: Align, The alignment of the new objectWORLD World, Align the new object to the world.VIEW View, Align the new object to the view.CURSOR 3D Cursor, Use the 3D cursor orientation for the new object. 
    :type align: typing.Union[int, str]
    :param location: Location, Location for the newly added object 
    :type location: float
    :param rotation: Rotation, Rotation for the newly added object 
    :type rotation: float
    '''

    pass


def collection_link(collection: typing.Union[int, str] = ''):
    '''Add an object to an existing collection 

    :param collection: Collection 
    :type collection: typing.Union[int, str]
    '''

    pass


def collection_objects_select():
    '''Select all objects in collection 

    '''

    pass


def collection_remove():
    '''Remove the active object from this collection 

    '''

    pass


def collection_unlink():
    '''Unlink the collection from all objects 

    '''

    pass


def constraint_add(type: typing.Union[int, str] = ''):
    '''Add a constraint to the active object 

    :param type: Type 
    :type type: typing.Union[int, str]
    '''

    pass


def constraint_add_with_targets(type: typing.Union[int, str] = ''):
    '''Add a constraint to the active object, with target (where applicable) set to the selected Objects/Bones 

    :param type: Type 
    :type type: typing.Union[int, str]
    '''

    pass


def constraints_clear():
    '''Clear all the constraints for the active Object only 

    '''

    pass


def constraints_copy():
    '''Copy constraints to other selected objects 

    '''

    pass


def convert(target: typing.Union[int, str] = 'MESH',
            keep_original: bool = False):
    '''Convert selected objects to another type 

    :param target: Target, Type of object to convert to 
    :type target: typing.Union[int, str]
    :param keep_original: Keep Original, Keep original objects instead of replacing them 
    :type keep_original: bool
    '''

    pass


def correctivesmooth_bind(modifier: str = ""):
    '''Bind base pose in Corrective Smooth modifier 

    :param modifier: Modifier, Name of the modifier to edit 
    :type modifier: str
    '''

    pass


def data_transfer(use_reverse_transfer: bool = False,
                  use_freeze: bool = False,
                  data_type: typing.Union[int, str] = '',
                  use_create: bool = True,
                  vert_mapping: typing.Union[int, str] = 'NEAREST',
                  edge_mapping: typing.Union[int, str] = 'NEAREST',
                  loop_mapping: typing.Union[int, str] = 'NEAREST_POLYNOR',
                  poly_mapping: typing.Union[int, str] = 'NEAREST',
                  use_auto_transform: bool = False,
                  use_object_transform: bool = True,
                  use_max_distance: bool = False,
                  max_distance: float = 1.0,
                  ray_radius: float = 0.0,
                  islands_precision: float = 0.1,
                  layers_select_src: typing.Union[int, str] = 'ACTIVE',
                  layers_select_dst: typing.Union[int, str] = 'ACTIVE',
                  mix_mode: typing.Union[int, str] = 'REPLACE',
                  mix_factor: float = 1.0):
    '''Transfer data layer(s) (weights, edge sharp, …) from active to selected meshes 

    :param use_reverse_transfer: Reverse Transfer, Transfer from selected objects to active one 
    :type use_reverse_transfer: bool
    :param use_freeze: Freeze Operator, Prevent changes to settings to re-run the operator, handy to change several things at once with heavy geometry 
    :type use_freeze: bool
    :param data_type: Data Type, Which data to transferVGROUP_WEIGHTS Vertex Group(s), Transfer active or all vertex groups.BEVEL_WEIGHT_VERT Bevel Weight, Transfer bevel weights.SHARP_EDGE Sharp, Transfer sharp mark.SEAM UV Seam, Transfer UV seam mark.CREASE Subsurf Crease, Transfer crease values.BEVEL_WEIGHT_EDGE Bevel Weight, Transfer bevel weights.FREESTYLE_EDGE Freestyle Mark, Transfer Freestyle edge mark.CUSTOM_NORMAL Custom Normals, Transfer custom normals.VCOL VCol, Vertex (face corners) colors.UV UVs, Transfer UV layers.SMOOTH Smooth, Transfer flat/smooth mark.FREESTYLE_FACE Freestyle Mark, Transfer Freestyle face mark. 
    :type data_type: typing.Union[int, str]
    :param use_create: Create Data, Add data layers on destination meshes if needed 
    :type use_create: bool
    :param vert_mapping: Vertex Mapping, Method used to map source vertices to destination onesTOPOLOGY Topology, Copy from identical topology meshes.NEAREST Nearest vertex, Copy from closest vertex.EDGE_NEAREST Nearest Edge Vertex, Copy from closest vertex of closest edge.EDGEINTERP_NEAREST Nearest Edge Interpolated, Copy from interpolated values of vertices from closest point on closest edge.POLY_NEAREST Nearest Face Vertex, Copy from closest vertex of closest face.POLYINTERP_NEAREST Nearest Face Interpolated, Copy from interpolated values of vertices from closest point on closest face.POLYINTERP_VNORPROJ Projected Face Interpolated, Copy from interpolated values of vertices from point on closest face hit by normal-projection. 
    :type vert_mapping: typing.Union[int, str]
    :param edge_mapping: Edge Mapping, Method used to map source edges to destination onesTOPOLOGY Topology, Copy from identical topology meshes.VERT_NEAREST Nearest Vertices, Copy from most similar edge (edge which vertices are the closest of destination edge’s ones).NEAREST Nearest Edge, Copy from closest edge (using midpoints).POLY_NEAREST Nearest Face Edge, Copy from closest edge of closest face (using midpoints).EDGEINTERP_VNORPROJ Projected Edge Interpolated, Interpolate all source edges hit by the projection of destination one along its own normal (from vertices). 
    :type edge_mapping: typing.Union[int, str]
    :param loop_mapping: Face Corner Mapping, Method used to map source faces’ corners to destination onesTOPOLOGY Topology, Copy from identical topology meshes.NEAREST_NORMAL Nearest Corner And Best Matching Normal, Copy from nearest corner which has the best matching normal.NEAREST_POLYNOR Nearest Corner And Best Matching Face Normal, Copy from nearest corner which has the face with the best matching normal to destination corner’s face one.NEAREST_POLY Nearest Corner Of Nearest Face, Copy from nearest corner of nearest polygon.POLYINTERP_NEAREST Nearest Face Interpolated, Copy from interpolated corners of the nearest source polygon.POLYINTERP_LNORPROJ Projected Face Interpolated, Copy from interpolated corners of the source polygon hit by corner normal projection. 
    :type loop_mapping: typing.Union[int, str]
    :param poly_mapping: Face Mapping, Method used to map source faces to destination onesTOPOLOGY Topology, Copy from identical topology meshes.NEAREST Nearest Face, Copy from nearest polygon (using center points).NORMAL Best Normal-Matching, Copy from source polygon which normal is the closest to destination one.POLYINTERP_PNORPROJ Projected Face Interpolated, Interpolate all source polygons intersected by the projection of destination one along its own normal. 
    :type poly_mapping: typing.Union[int, str]
    :param use_auto_transform: Auto Transform, Automatically compute transformation to get the best possible match between source and destination meshes (WARNING: results will never be as good as manual matching of objects) 
    :type use_auto_transform: bool
    :param use_object_transform: Object Transform, Evaluate source and destination meshes in global space 
    :type use_object_transform: bool
    :param use_max_distance: Only Neighbor Geometry, Source elements must be closer than given distance from destination one 
    :type use_max_distance: bool
    :param max_distance: Max Distance, Maximum allowed distance between source and destination element, for non-topology mappings 
    :type max_distance: float
    :param ray_radius: Ray Radius, ‘Width’ of rays (especially useful when raycasting against vertices or edges) 
    :type ray_radius: float
    :param islands_precision: Islands Precision, Factor controlling precision of islands handling (the higher, the better the results) 
    :type islands_precision: float
    :param layers_select_src: Source Layers Selection, Which layers to transfer, in case of multi-layers typesACTIVE Active Layer, Only transfer active data layer.ALL All Layers, Transfer all data layers.BONE_SELECT Selected Pose Bones, Transfer all vertex groups used by selected pose bones.BONE_DEFORM Deform Pose Bones, Transfer all vertex groups used by deform bones. 
    :type layers_select_src: typing.Union[int, str]
    :param layers_select_dst: Destination Layers Matching, How to match source and destination layersACTIVE Active Layer, Affect active data layer of all targets.NAME By Name, Match target data layers to affect by name.INDEX By Order, Match target data layers to affect by order (indices). 
    :type layers_select_dst: typing.Union[int, str]
    :param mix_mode: Mix Mode, How to affect destination elements with source valuesREPLACE Replace, Overwrite all elements’ data.ABOVE_THRESHOLD Above Threshold, Only replace destination elements where data is above given threshold (exact behavior depends on data type).BELOW_THRESHOLD Below Threshold, Only replace destination elements where data is below given threshold (exact behavior depends on data type).MIX Mix, Mix source value into destination one, using given threshold as factor.ADD Add, Add source value to destination one, using given threshold as factor.SUB Subtract, Subtract source value to destination one, using given threshold as factor.MUL Multiply, Multiply source value to destination one, using given threshold as factor. 
    :type mix_mode: typing.Union[int, str]
    :param mix_factor: Mix Factor, Factor to use when applying data to destination (exact behavior depends on mix mode) 
    :type mix_factor: float
    '''

    pass


def datalayout_transfer(modifier: str = "",
                        data_type: typing.Union[int, str] = '',
                        use_delete: bool = False,
                        layers_select_src: typing.Union[int, str] = 'ACTIVE',
                        layers_select_dst: typing.Union[int, str] = 'ACTIVE'):
    '''Transfer layout of data layer(s) from active to selected meshes 

    :param modifier: Modifier, Name of the modifier to edit 
    :type modifier: str
    :param data_type: Data Type, Which data to transferVGROUP_WEIGHTS Vertex Group(s), Transfer active or all vertex groups.BEVEL_WEIGHT_VERT Bevel Weight, Transfer bevel weights.SHARP_EDGE Sharp, Transfer sharp mark.SEAM UV Seam, Transfer UV seam mark.CREASE Subsurf Crease, Transfer crease values.BEVEL_WEIGHT_EDGE Bevel Weight, Transfer bevel weights.FREESTYLE_EDGE Freestyle Mark, Transfer Freestyle edge mark.CUSTOM_NORMAL Custom Normals, Transfer custom normals.VCOL VCol, Vertex (face corners) colors.UV UVs, Transfer UV layers.SMOOTH Smooth, Transfer flat/smooth mark.FREESTYLE_FACE Freestyle Mark, Transfer Freestyle face mark. 
    :type data_type: typing.Union[int, str]
    :param use_delete: Exact Match, Also delete some data layers from destination if necessary, so that it matches exactly source 
    :type use_delete: bool
    :param layers_select_src: Source Layers Selection, Which layers to transfer, in case of multi-layers typesACTIVE Active Layer, Only transfer active data layer.ALL All Layers, Transfer all data layers.BONE_SELECT Selected Pose Bones, Transfer all vertex groups used by selected pose bones.BONE_DEFORM Deform Pose Bones, Transfer all vertex groups used by deform bones. 
    :type layers_select_src: typing.Union[int, str]
    :param layers_select_dst: Destination Layers Matching, How to match source and destination layersACTIVE Active Layer, Affect active data layer of all targets.NAME By Name, Match target data layers to affect by name.INDEX By Order, Match target data layers to affect by order (indices). 
    :type layers_select_dst: typing.Union[int, str]
    '''

    pass


def delete(use_global: bool = False, confirm: bool = True):
    '''Delete selected objects 

    :param use_global: Delete Globally, Remove object from all scenes 
    :type use_global: bool
    :param confirm: Confirm, Prompt for confirmation 
    :type confirm: bool
    '''

    pass


def drop_named_image(filepath: str = "",
                     relative_path: bool = True,
                     name: str = "",
                     align: typing.Union[int, str] = 'WORLD',
                     location: float = (0.0, 0.0, 0.0),
                     rotation: float = (0.0, 0.0, 0.0)):
    '''Add an empty image type to scene with data 

    :param filepath: Filepath, Path to image file 
    :type filepath: str
    :param relative_path: Relative Path, Select the file relative to the blend file 
    :type relative_path: bool
    :param name: Name, Image name to assign 
    :type name: str
    :param align: Align, The alignment of the new objectWORLD World, Align the new object to the world.VIEW View, Align the new object to the view.CURSOR 3D Cursor, Use the 3D cursor orientation for the new object. 
    :type align: typing.Union[int, str]
    :param location: Location, Location for the newly added object 
    :type location: float
    :param rotation: Rotation, Rotation for the newly added object 
    :type rotation: float
    '''

    pass


def drop_named_material(name: str = "Material"):
    '''Undocumented contribute <https://developer.blender.org/T51061> 

    :param name: Name, Material name to assign 
    :type name: str
    '''

    pass


def duplicate(linked: bool = False,
              mode: typing.Union[int, str] = 'TRANSLATION'):
    '''Duplicate selected objects 

    :param linked: Linked, Duplicate object but not object data, linking to the original data 
    :type linked: bool
    :param mode: Mode 
    :type mode: typing.Union[int, str]
    '''

    pass


def duplicate_move(OBJECT_OT_duplicate=None, TRANSFORM_OT_translate=None):
    '''Duplicate selected objects and move them 

    :param OBJECT_OT_duplicate: Duplicate Objects, Duplicate selected objects 
    :param TRANSFORM_OT_translate: Move, Move selected items 
    '''

    pass


def duplicate_move_linked(OBJECT_OT_duplicate=None,
                          TRANSFORM_OT_translate=None):
    '''Duplicate selected objects and move them 

    :param OBJECT_OT_duplicate: Duplicate Objects, Duplicate selected objects 
    :param TRANSFORM_OT_translate: Move, Move selected items 
    '''

    pass


def duplicates_make_real(use_base_parent: bool = False,
                         use_hierarchy: bool = False):
    '''Make instanced objects attached to this object real 

    :param use_base_parent: Parent, Parent newly created objects to the original duplicator 
    :type use_base_parent: bool
    :param use_hierarchy: Keep Hierarchy, Maintain parent child relationships 
    :type use_hierarchy: bool
    '''

    pass


def editmode_toggle():
    '''Toggle object’s editmode 

    '''

    pass


def effector_add(type: typing.Union[int, str] = 'FORCE',
                 radius: float = 1.0,
                 enter_editmode: bool = False,
                 align: typing.Union[int, str] = 'WORLD',
                 location: float = (0.0, 0.0, 0.0),
                 rotation: float = (0.0, 0.0, 0.0)):
    '''Add an empty object with a physics effector to the scene 

    :param type: Type 
    :type type: typing.Union[int, str]
    :param radius: Radius 
    :type radius: float
    :param enter_editmode: Enter Editmode, Enter editmode when adding this object 
    :type enter_editmode: bool
    :param align: Align, The alignment of the new objectWORLD World, Align the new object to the world.VIEW View, Align the new object to the view.CURSOR 3D Cursor, Use the 3D cursor orientation for the new object. 
    :type align: typing.Union[int, str]
    :param location: Location, Location for the newly added object 
    :type location: float
    :param rotation: Rotation, Rotation for the newly added object 
    :type rotation: float
    '''

    pass


def empty_add(type: typing.Union[int, str] = 'PLAIN_AXES',
              radius: float = 1.0,
              align: typing.Union[int, str] = 'WORLD',
              location: float = (0.0, 0.0, 0.0),
              rotation: float = (0.0, 0.0, 0.0)):
    '''Add an empty object to the scene 

    :param type: Type 
    :type type: typing.Union[int, str]
    :param radius: Radius 
    :type radius: float
    :param align: Align, The alignment of the new objectWORLD World, Align the new object to the world.VIEW View, Align the new object to the view.CURSOR 3D Cursor, Use the 3D cursor orientation for the new object. 
    :type align: typing.Union[int, str]
    :param location: Location, Location for the newly added object 
    :type location: float
    :param rotation: Rotation, Rotation for the newly added object 
    :type rotation: float
    '''

    pass


def explode_refresh(modifier: str = ""):
    '''Refresh data in the Explode modifier 

    :param modifier: Modifier, Name of the modifier to edit 
    :type modifier: str
    '''

    pass


def face_map_add():
    '''Add a new face map to the active object 

    '''

    pass


def face_map_assign():
    '''Assign faces to a face map 

    '''

    pass


def face_map_deselect():
    '''Deselect faces belonging to a face map 

    '''

    pass


def face_map_move(direction: typing.Union[int, str] = 'UP'):
    '''Move the active face map up/down in the list 

    :param direction: Direction, Direction to move, UP or DOWN 
    :type direction: typing.Union[int, str]
    '''

    pass


def face_map_remove():
    '''Remove a face map from the active object 

    '''

    pass


def face_map_remove_from():
    '''Remove faces from a face map 

    '''

    pass


def face_map_select():
    '''Select faces belonging to a face map 

    '''

    pass


def forcefield_toggle():
    '''Toggle object’s force field 

    '''

    pass


def gpencil_add(radius: float = 1.0,
                align: typing.Union[int, str] = 'WORLD',
                location: float = (0.0, 0.0, 0.0),
                rotation: float = (0.0, 0.0, 0.0),
                type: typing.Union[int, str] = 'EMPTY'):
    '''Add a Grease Pencil object to the scene 

    :param radius: Radius 
    :type radius: float
    :param align: Align, The alignment of the new objectWORLD World, Align the new object to the world.VIEW View, Align the new object to the view.CURSOR 3D Cursor, Use the 3D cursor orientation for the new object. 
    :type align: typing.Union[int, str]
    :param location: Location, Location for the newly added object 
    :type location: float
    :param rotation: Rotation, Rotation for the newly added object 
    :type rotation: float
    :param type: TypeEMPTY Blank, Create an empty grease pencil object.STROKE Stroke, Create a simple stroke with basic colors.MONKEY Monkey, Construct a Suzanne grease pencil object. 
    :type type: typing.Union[int, str]
    '''

    pass


def gpencil_modifier_add(type: typing.Union[int, str] = 'CURVE'):
    '''Add a procedural operation/effect to the active grease pencil object 

    :param type: TypeDATA_TRANSFER Data Transfer.MESH_CACHE Mesh Cache.MESH_SEQUENCE_CACHE Mesh Sequence Cache.NORMAL_EDIT Normal Edit.WEIGHTED_NORMAL Weighted Normal.UV_PROJECT UV Project.UV_WARP UV Warp.VERTEX_WEIGHT_EDIT Vertex Weight Edit.VERTEX_WEIGHT_MIX Vertex Weight Mix.VERTEX_WEIGHT_PROXIMITY Vertex Weight Proximity.ARRAY Array.BEVEL Bevel.BOOLEAN Boolean.BUILD Build.DECIMATE Decimate.EDGE_SPLIT Edge Split.MASK Mask.MIRROR Mirror.MULTIRES Multiresolution.REMESH Remesh.SCREW Screw.SKIN Skin.SOLIDIFY Solidify.SUBSURF Subdivision Surface.TRIANGULATE Triangulate.WIREFRAME Wireframe, Generate a wireframe on the edges of a mesh.ARMATURE Armature.CAST Cast.CURVE Curve.DISPLACE Displace.HOOK Hook.LAPLACIANDEFORM Laplacian Deform.LATTICE Lattice.MESH_DEFORM Mesh Deform.SHRINKWRAP Shrinkwrap.SIMPLE_DEFORM Simple Deform.SMOOTH Smooth.CORRECTIVE_SMOOTH Smooth Corrective.LAPLACIANSMOOTH Smooth Laplacian.SURFACE_DEFORM Surface Deform.WARP Warp.WAVE Wave.CLOTH Cloth.COLLISION Collision.DYNAMIC_PAINT Dynamic Paint.EXPLODE Explode.FLUID_SIMULATION Fluid Simulation.OCEAN Ocean.PARTICLE_INSTANCE Particle Instance.PARTICLE_SYSTEM Particle System.SMOKE Smoke.SOFT_BODY Soft Body.SURFACE Surface. 
    :type type: typing.Union[int, str]
    '''

    pass


def gpencil_modifier_apply(apply_as: typing.Union[int, str] = 'DATA',
                           modifier: str = ""):
    '''Apply modifier and remove from the stack 

    :param apply_as: Apply as, How to apply the modifier to the geometryDATA Object Data, Apply modifier to the object’s data.SHAPE New Shape, Apply deform-only modifier to a new shape on this object. 
    :type apply_as: typing.Union[int, str]
    :param modifier: Modifier, Name of the modifier to edit 
    :type modifier: str
    '''

    pass


def gpencil_modifier_copy(modifier: str = ""):
    '''Duplicate modifier at the same position in the stack 

    :param modifier: Modifier, Name of the modifier to edit 
    :type modifier: str
    '''

    pass


def gpencil_modifier_move_down(modifier: str = ""):
    '''Move modifier down in the stack 

    :param modifier: Modifier, Name of the modifier to edit 
    :type modifier: str
    '''

    pass


def gpencil_modifier_move_up(modifier: str = ""):
    '''Move modifier up in the stack 

    :param modifier: Modifier, Name of the modifier to edit 
    :type modifier: str
    '''

    pass


def gpencil_modifier_remove(modifier: str = ""):
    '''Remove a modifier from the active grease pencil object 

    :param modifier: Modifier, Name of the modifier to edit 
    :type modifier: str
    '''

    pass


def hide_collection(collection_index: int = -1, toggle: bool = False):
    '''Show only objects in collection (Shift to extend) 

    :param collection_index: Collection Index, Index of the collection to change visibility 
    :type collection_index: int
    :param toggle: Toggle, Toggle visibility 
    :type toggle: bool
    '''

    pass


def hide_render_clear_all():
    '''Reveal all render objects by setting the hide render flag 

    '''

    pass


def hide_view_clear(select: bool = True):
    '''Reveal temporarily hidden objects 

    :param select: Select 
    :type select: bool
    '''

    pass


def hide_view_set(unselected: bool = False):
    '''Temporarily hide objects from the viewport 

    :param unselected: Unselected, Hide unselected rather than selected objects 
    :type unselected: bool
    '''

    pass


def hook_add_newob():
    '''Hook selected vertices to a newly created object 

    '''

    pass


def hook_add_selob(use_bone: bool = False):
    '''Hook selected vertices to the first selected object 

    :param use_bone: Active Bone, Assign the hook to the hook objects active bone 
    :type use_bone: bool
    '''

    pass


def hook_assign(modifier: typing.Union[int, str] = ''):
    '''Assign the selected vertices to a hook 

    :param modifier: Modifier, Modifier number to assign to 
    :type modifier: typing.Union[int, str]
    '''

    pass


def hook_recenter(modifier: typing.Union[int, str] = ''):
    '''Set hook center to cursor position 

    :param modifier: Modifier, Modifier number to assign to 
    :type modifier: typing.Union[int, str]
    '''

    pass


def hook_remove(modifier: typing.Union[int, str] = ''):
    '''Remove a hook from the active object 

    :param modifier: Modifier, Modifier number to remove 
    :type modifier: typing.Union[int, str]
    '''

    pass


def hook_reset(modifier: typing.Union[int, str] = ''):
    '''Recalculate and clear offset transformation 

    :param modifier: Modifier, Modifier number to assign to 
    :type modifier: typing.Union[int, str]
    '''

    pass


def hook_select(modifier: typing.Union[int, str] = ''):
    '''Select affected vertices on mesh 

    :param modifier: Modifier, Modifier number to remove 
    :type modifier: typing.Union[int, str]
    '''

    pass


def instance_offset_from_cursor():
    '''Set offset used for collection instances based on cursor position 

    '''

    pass


def isolate_type_render():
    '''Hide unselected render objects of same type as active by setting the hide render flag 

    '''

    pass


def join():
    '''Join selected objects into active object 

    '''

    pass


def join_shapes():
    '''Copy the current resulting shape of another selected object to this one 

    '''

    pass


def join_uvs():
    '''Transfer UV Maps from active to selected objects (needs matching geometry) 

    '''

    pass


def laplaciandeform_bind(modifier: str = ""):
    '''Bind mesh to system in laplacian deform modifier 

    :param modifier: Modifier, Name of the modifier to edit 
    :type modifier: str
    '''

    pass


def light_add(type: typing.Union[int, str] = 'POINT',
              radius: float = 1.0,
              align: typing.Union[int, str] = 'WORLD',
              location: float = (0.0, 0.0, 0.0),
              rotation: float = (0.0, 0.0, 0.0)):
    '''Add a light object to the scene 

    :param type: TypePOINT Point, Omnidirectional point light source.SUN Sun, Constant direction parallel ray light source.SPOT Spot, Directional cone light source.AREA Area, Directional area light source. 
    :type type: typing.Union[int, str]
    :param radius: Radius 
    :type radius: float
    :param align: Align, The alignment of the new objectWORLD World, Align the new object to the world.VIEW View, Align the new object to the view.CURSOR 3D Cursor, Use the 3D cursor orientation for the new object. 
    :type align: typing.Union[int, str]
    :param location: Location, Location for the newly added object 
    :type location: float
    :param rotation: Rotation, Rotation for the newly added object 
    :type rotation: float
    '''

    pass


def lightprobe_add(type: typing.Union[int, str] = 'CUBEMAP',
                   radius: float = 1.0,
                   enter_editmode: bool = False,
                   align: typing.Union[int, str] = 'WORLD',
                   location: float = (0.0, 0.0, 0.0),
                   rotation: float = (0.0, 0.0, 0.0)):
    '''Add a light probe object 

    :param type: TypeCUBEMAP Reflection Cubemap, Reflection probe with spherical or cubic attenuation.PLANAR Reflection Plane, Planar reflection probe.GRID Irradiance Volume, Irradiance probe to capture diffuse indirect lighting. 
    :type type: typing.Union[int, str]
    :param radius: Radius 
    :type radius: float
    :param enter_editmode: Enter Editmode, Enter editmode when adding this object 
    :type enter_editmode: bool
    :param align: Align, The alignment of the new objectWORLD World, Align the new object to the world.VIEW View, Align the new object to the view.CURSOR 3D Cursor, Use the 3D cursor orientation for the new object. 
    :type align: typing.Union[int, str]
    :param location: Location, Location for the newly added object 
    :type location: float
    :param rotation: Rotation, Rotation for the newly added object 
    :type rotation: float
    '''

    pass


def link_to_collection(collection_index: int = -1,
                       is_new: bool = False,
                       new_collection_name: str = ""):
    '''Link objects to a collection 

    :param collection_index: Collection Index, Index of the collection to move to 
    :type collection_index: int
    :param is_new: New, Move objects to a new collection 
    :type is_new: bool
    :param new_collection_name: Name, Name of the newly added collection 
    :type new_collection_name: str
    '''

    pass


def load_background_image(filepath: str = "",
                          filter_image: bool = True,
                          filter_folder: bool = True,
                          view_align: bool = True):
    '''Add a reference image into the background behind objects 

    :param filepath: filepath 
    :type filepath: str
    :param filter_image: filter_image 
    :type filter_image: bool
    :param filter_folder: filter_folder 
    :type filter_folder: bool
    :param view_align: Align to view 
    :type view_align: bool
    '''

    pass


def load_reference_image(filepath: str = "",
                         filter_image: bool = True,
                         filter_folder: bool = True,
                         view_align: bool = True):
    '''Add a reference image into the scene between objects 

    :param filepath: filepath 
    :type filepath: str
    :param filter_image: filter_image 
    :type filter_image: bool
    :param filter_folder: filter_folder 
    :type filter_folder: bool
    :param view_align: Align to view 
    :type view_align: bool
    '''

    pass


def location_clear(clear_delta: bool = False):
    '''Clear the object’s location 

    :param clear_delta: Clear Delta, Clear delta location in addition to clearing the normal location transform 
    :type clear_delta: bool
    '''

    pass


def make_dupli_face():
    '''Convert objects into instanced faces 

    '''

    pass


def make_links_data(type: typing.Union[int, str] = 'OBDATA'):
    '''Apply active object links to other selected objects 

    :param type: Type 
    :type type: typing.Union[int, str]
    '''

    pass


def make_links_scene(scene: typing.Union[int, str] = ''):
    '''Link selection to another scene 

    :param scene: Scene 
    :type scene: typing.Union[int, str]
    '''

    pass


def make_local(type: typing.Union[int, str] = 'SELECT_OBJECT'):
    '''Make library linked data-blocks local to this file 

    :param type: Type 
    :type type: typing.Union[int, str]
    '''

    pass


def make_override_library(object: typing.Union[int, str] = 'DEFAULT'):
    '''Make a local override of this library linked data-block 

    :param object: Override Object, Name of lib-linked/collection object to make an override from 
    :type object: typing.Union[int, str]
    '''

    pass


def make_single_user(type: typing.Union[int, str] = 'SELECTED_OBJECTS',
                     object: bool = False,
                     obdata: bool = False,
                     material: bool = False,
                     animation: bool = False):
    '''Make linked data local to each object 

    :param type: Type 
    :type type: typing.Union[int, str]
    :param object: Object, Make single user objects 
    :type object: bool
    :param obdata: Object Data, Make single user object data 
    :type obdata: bool
    :param material: Materials, Make materials local to each data-block 
    :type material: bool
    :param animation: Object Animation, Make animation data local to each object 
    :type animation: bool
    '''

    pass


def material_slot_add():
    '''Add a new material slot 

    '''

    pass


def material_slot_assign():
    '''Assign active material slot to selection 

    '''

    pass


def material_slot_copy():
    '''Copy material to selected objects 

    '''

    pass


def material_slot_deselect():
    '''Deselect by active material slot 

    '''

    pass


def material_slot_move(direction: typing.Union[int, str] = 'UP'):
    '''Move the active material up/down in the list 

    :param direction: Direction, Direction to move the active material towards 
    :type direction: typing.Union[int, str]
    '''

    pass


def material_slot_remove():
    '''Remove the selected material slot 

    '''

    pass


def material_slot_select():
    '''Select by active material slot 

    '''

    pass


def meshdeform_bind(modifier: str = ""):
    '''Bind mesh to cage in mesh deform modifier 

    :param modifier: Modifier, Name of the modifier to edit 
    :type modifier: str
    '''

    pass


def metaball_add(type: typing.Union[int, str] = 'BALL',
                 radius: float = 1.0,
                 enter_editmode: bool = False,
                 align: typing.Union[int, str] = 'WORLD',
                 location: float = (0.0, 0.0, 0.0),
                 rotation: float = (0.0, 0.0, 0.0)):
    '''Add an metaball object to the scene 

    :param type: Primitive 
    :type type: typing.Union[int, str]
    :param radius: Radius 
    :type radius: float
    :param enter_editmode: Enter Editmode, Enter editmode when adding this object 
    :type enter_editmode: bool
    :param align: Align, The alignment of the new objectWORLD World, Align the new object to the world.VIEW View, Align the new object to the view.CURSOR 3D Cursor, Use the 3D cursor orientation for the new object. 
    :type align: typing.Union[int, str]
    :param location: Location, Location for the newly added object 
    :type location: float
    :param rotation: Rotation, Rotation for the newly added object 
    :type rotation: float
    '''

    pass


def mode_set(mode: typing.Union[int, str] = 'OBJECT', toggle: bool = False):
    '''Sets the object interaction mode 

    :param mode: ModeOBJECT Object Mode.EDIT Edit Mode.POSE Pose Mode.SCULPT Sculpt Mode.VERTEX_PAINT Vertex Paint.WEIGHT_PAINT Weight Paint.TEXTURE_PAINT Texture Paint.PARTICLE_EDIT Particle Edit.EDIT_GPENCIL Edit Mode, Edit Grease Pencil Strokes.SCULPT_GPENCIL Sculpt Mode, Sculpt Grease Pencil Strokes.PAINT_GPENCIL Draw, Paint Grease Pencil Strokes.WEIGHT_GPENCIL Weight Paint, Grease Pencil Weight Paint Strokes. 
    :type mode: typing.Union[int, str]
    :param toggle: Toggle 
    :type toggle: bool
    '''

    pass


def mode_set_or_submode(mode: typing.Union[int, str] = 'OBJECT',
                        toggle: bool = False):
    '''Sets the object interaction mode 

    :param mode: ModeOBJECT Object Mode.EDIT Edit Mode.POSE Pose Mode.SCULPT Sculpt Mode.VERTEX_PAINT Vertex Paint.WEIGHT_PAINT Weight Paint.TEXTURE_PAINT Texture Paint.PARTICLE_EDIT Particle Edit.EDIT_GPENCIL Edit Mode, Edit Grease Pencil Strokes.SCULPT_GPENCIL Sculpt Mode, Sculpt Grease Pencil Strokes.PAINT_GPENCIL Draw, Paint Grease Pencil Strokes.WEIGHT_GPENCIL Weight Paint, Grease Pencil Weight Paint Strokes. 
    :type mode: typing.Union[int, str]
    :param toggle: Toggle 
    :type toggle: bool
    '''

    pass


def modifier_add(type: typing.Union[int, str] = 'SUBSURF'):
    '''Add a procedural operation/effect to the active object 

    :param type: TypeDATA_TRANSFER Data Transfer.MESH_CACHE Mesh Cache.MESH_SEQUENCE_CACHE Mesh Sequence Cache.NORMAL_EDIT Normal Edit.WEIGHTED_NORMAL Weighted Normal.UV_PROJECT UV Project.UV_WARP UV Warp.VERTEX_WEIGHT_EDIT Vertex Weight Edit.VERTEX_WEIGHT_MIX Vertex Weight Mix.VERTEX_WEIGHT_PROXIMITY Vertex Weight Proximity.ARRAY Array.BEVEL Bevel.BOOLEAN Boolean.BUILD Build.DECIMATE Decimate.EDGE_SPLIT Edge Split.MASK Mask.MIRROR Mirror.MULTIRES Multiresolution.REMESH Remesh.SCREW Screw.SKIN Skin.SOLIDIFY Solidify.SUBSURF Subdivision Surface.TRIANGULATE Triangulate.WIREFRAME Wireframe, Generate a wireframe on the edges of a mesh.ARMATURE Armature.CAST Cast.CURVE Curve.DISPLACE Displace.HOOK Hook.LAPLACIANDEFORM Laplacian Deform.LATTICE Lattice.MESH_DEFORM Mesh Deform.SHRINKWRAP Shrinkwrap.SIMPLE_DEFORM Simple Deform.SMOOTH Smooth.CORRECTIVE_SMOOTH Smooth Corrective.LAPLACIANSMOOTH Smooth Laplacian.SURFACE_DEFORM Surface Deform.WARP Warp.WAVE Wave.CLOTH Cloth.COLLISION Collision.DYNAMIC_PAINT Dynamic Paint.EXPLODE Explode.FLUID_SIMULATION Fluid Simulation.OCEAN Ocean.PARTICLE_INSTANCE Particle Instance.PARTICLE_SYSTEM Particle System.SMOKE Smoke.SOFT_BODY Soft Body.SURFACE Surface. 
    :type type: typing.Union[int, str]
    '''

    pass


def modifier_apply(apply_as: typing.Union[int, str] = 'DATA',
                   modifier: str = ""):
    '''Apply modifier and remove from the stack 

    :param apply_as: Apply as, How to apply the modifier to the geometryDATA Object Data, Apply modifier to the object’s data.SHAPE New Shape, Apply deform-only modifier to a new shape on this object. 
    :type apply_as: typing.Union[int, str]
    :param modifier: Modifier, Name of the modifier to edit 
    :type modifier: str
    '''

    pass


def modifier_convert(modifier: str = ""):
    '''Convert particles to a mesh object 

    :param modifier: Modifier, Name of the modifier to edit 
    :type modifier: str
    '''

    pass


def modifier_copy(modifier: str = ""):
    '''Duplicate modifier at the same position in the stack 

    :param modifier: Modifier, Name of the modifier to edit 
    :type modifier: str
    '''

    pass


def modifier_move_down(modifier: str = ""):
    '''Move modifier down in the stack 

    :param modifier: Modifier, Name of the modifier to edit 
    :type modifier: str
    '''

    pass


def modifier_move_up(modifier: str = ""):
    '''Move modifier up in the stack 

    :param modifier: Modifier, Name of the modifier to edit 
    :type modifier: str
    '''

    pass


def modifier_remove(modifier: str = ""):
    '''Remove a modifier from the active object 

    :param modifier: Modifier, Name of the modifier to edit 
    :type modifier: str
    '''

    pass


def move_to_collection(collection_index: int = -1,
                       is_new: bool = False,
                       new_collection_name: str = ""):
    '''Move objects to a collection 

    :param collection_index: Collection Index, Index of the collection to move to 
    :type collection_index: int
    :param is_new: New, Move objects to a new collection 
    :type is_new: bool
    :param new_collection_name: Name, Name of the newly added collection 
    :type new_collection_name: str
    '''

    pass


def multires_base_apply(modifier: str = ""):
    '''Modify the base mesh to conform to the displaced mesh 

    :param modifier: Modifier, Name of the modifier to edit 
    :type modifier: str
    '''

    pass


def multires_external_pack():
    '''Pack displacements from an external file 

    '''

    pass


def multires_external_save(
        filepath: str = "",
        check_existing: bool = True,
        filter_blender: bool = False,
        filter_backup: bool = False,
        filter_image: bool = False,
        filter_movie: bool = False,
        filter_python: bool = False,
        filter_font: bool = False,
        filter_sound: bool = False,
        filter_text: bool = False,
        filter_btx: bool = True,
        filter_collada: bool = False,
        filter_alembic: bool = False,
        filter_folder: bool = True,
        filter_blenlib: bool = False,
        filemode: int = 9,
        relative_path: bool = True,
        display_type: typing.Union[int, str] = 'DEFAULT',
        sort_method: typing.Union[int, str] = 'FILE_SORT_ALPHA',
        modifier: str = ""):
    '''Save displacements to an external file 

    :param filepath: File Path, Path to file 
    :type filepath: str
    :param check_existing: Check Existing, Check and warn on overwriting existing files 
    :type check_existing: bool
    :param filter_blender: Filter .blend files 
    :type filter_blender: bool
    :param filter_backup: Filter .blend files 
    :type filter_backup: bool
    :param filter_image: Filter image files 
    :type filter_image: bool
    :param filter_movie: Filter movie files 
    :type filter_movie: bool
    :param filter_python: Filter python files 
    :type filter_python: bool
    :param filter_font: Filter font files 
    :type filter_font: bool
    :param filter_sound: Filter sound files 
    :type filter_sound: bool
    :param filter_text: Filter text files 
    :type filter_text: bool
    :param filter_btx: Filter btx files 
    :type filter_btx: bool
    :param filter_collada: Filter COLLADA files 
    :type filter_collada: bool
    :param filter_alembic: Filter Alembic files 
    :type filter_alembic: bool
    :param filter_folder: Filter folders 
    :type filter_folder: bool
    :param filter_blenlib: Filter Blender IDs 
    :type filter_blenlib: bool
    :param filemode: File Browser Mode, The setting for the file browser mode to load a .blend file, a library or a special file 
    :type filemode: int
    :param relative_path: Relative Path, Select the file relative to the blend file 
    :type relative_path: bool
    :param display_type: Display TypeDEFAULT Default, Automatically determine display type for files.LIST_SHORT Short List, Display files as short list.LIST_LONG Long List, Display files as a detailed list.THUMBNAIL Thumbnails, Display files as thumbnails. 
    :type display_type: typing.Union[int, str]
    :param sort_method: File sorting modeFILE_SORT_ALPHA Sort alphabetically, Sort the file list alphabetically.FILE_SORT_EXTENSION Sort by extension, Sort the file list by extension/type.FILE_SORT_TIME Sort by time, Sort files by modification time.FILE_SORT_SIZE Sort by size, Sort files by size. 
    :type sort_method: typing.Union[int, str]
    :param modifier: Modifier, Name of the modifier to edit 
    :type modifier: str
    '''

    pass


def multires_higher_levels_delete(modifier: str = ""):
    '''Deletes the higher resolution mesh, potential loss of detail 

    :param modifier: Modifier, Name of the modifier to edit 
    :type modifier: str
    '''

    pass


def multires_reshape(modifier: str = ""):
    '''Copy vertex coordinates from other object 

    :param modifier: Modifier, Name of the modifier to edit 
    :type modifier: str
    '''

    pass


def multires_subdivide(modifier: str = ""):
    '''Add a new level of subdivision 

    :param modifier: Modifier, Name of the modifier to edit 
    :type modifier: str
    '''

    pass


def ocean_bake(modifier: str = "", free: bool = False):
    '''Bake an image sequence of ocean data 

    :param modifier: Modifier, Name of the modifier to edit 
    :type modifier: str
    :param free: Free, Free the bake, rather than generating it 
    :type free: bool
    '''

    pass


def origin_clear():
    '''Clear the object’s origin 

    '''

    pass


def origin_set(type: typing.Union[int, str] = 'GEOMETRY_ORIGIN',
               center: typing.Union[int, str] = 'MEDIAN'):
    '''Set the object’s origin, by either moving the data, or set to center of data, or use 3D cursor 

    :param type: TypeGEOMETRY_ORIGIN Geometry to Origin, Move object geometry to object origin.ORIGIN_GEOMETRY Origin to Geometry, Calculate the center of geometry based on the current pivot point (median, otherwise bounding-box).ORIGIN_CURSOR Origin to 3D Cursor, Move object origin to position of the 3D cursor.ORIGIN_CENTER_OF_MASS Origin to Center of Mass (Surface), Calculate the center of mass from the surface area.ORIGIN_CENTER_OF_VOLUME Origin to Center of Mass (Volume), Calculate the center of mass from the volume (must be manifold geometry with consistent normals). 
    :type type: typing.Union[int, str]
    :param center: Center 
    :type center: typing.Union[int, str]
    '''

    pass


def parent_clear(type: typing.Union[int, str] = 'CLEAR'):
    '''Clear the object’s parenting 

    :param type: TypeCLEAR Clear Parent, Completely clear the parenting relationship, including involved modifiers if any.CLEAR_KEEP_TRANSFORM Clear and Keep Transformation, As ‘Clear Parent’, but keep the current visual transformations of the object.CLEAR_INVERSE Clear Parent Inverse, Reset the transform corrections applied to the parenting relationship, does not remove parenting itself. 
    :type type: typing.Union[int, str]
    '''

    pass


def parent_no_inverse_set():
    '''Set the object’s parenting without setting the inverse parent correction 

    '''

    pass


def parent_set(type: typing.Union[int, str] = 'OBJECT',
               xmirror: bool = False,
               keep_transform: bool = False):
    '''Set the object’s parenting 

    :param type: Type 
    :type type: typing.Union[int, str]
    :param xmirror: X Mirror, Apply weights symmetrically along X axis, for Envelope/Automatic vertex groups creation 
    :type xmirror: bool
    :param keep_transform: Keep Transform, Apply transformation before parenting 
    :type keep_transform: bool
    '''

    pass


def particle_system_add():
    '''Add a particle system 

    '''

    pass


def particle_system_remove():
    '''Remove the selected particle system 

    '''

    pass


def paths_calculate(start_frame: int = 1, end_frame: int = 250):
    '''Calculate motion paths for the selected objects 

    :param start_frame: Start, First frame to calculate object paths on 
    :type start_frame: int
    :param end_frame: End, Last frame to calculate object paths on 
    :type end_frame: int
    '''

    pass


def paths_clear(only_selected: bool = False):
    '''Clear path caches for all objects, hold Shift key for selected objects only 

    :param only_selected: Only Selected, Only clear paths from selected objects 
    :type only_selected: bool
    '''

    pass


def paths_range_update():
    '''Update frame range for motion paths from the Scene’s current frame range 

    '''

    pass


def paths_update():
    '''Recalculate paths for selected objects 

    '''

    pass


def posemode_toggle():
    '''Enable or disable posing/selecting bones 

    '''

    pass


def proxy_make(object: typing.Union[int, str] = 'DEFAULT'):
    '''Add empty object to become local replacement data of a library-linked object 

    :param object: Proxy Object, Name of lib-linked/collection object to make a proxy for 
    :type object: typing.Union[int, str]
    '''

    pass


def quick_explode(style: typing.Union[int, str] = 'EXPLODE',
                  amount: int = 100,
                  frame_duration: int = 50,
                  frame_start: int = 1,
                  frame_end: int = 10,
                  velocity: float = 1.0,
                  fade: bool = True):
    '''Make selected objects explode 

    :param style: Explode Style 
    :type style: typing.Union[int, str]
    :param amount: Amount of pieces 
    :type amount: int
    :param frame_duration: Duration 
    :type frame_duration: int
    :param frame_start: Start Frame 
    :type frame_start: int
    :param frame_end: End Frame 
    :type frame_end: int
    :param velocity: Outwards Velocity 
    :type velocity: float
    :param fade: Fade, Fade the pieces over time 
    :type fade: bool
    '''

    pass


def quick_fluid(style: typing.Union[int, str] = 'BASIC',
                initial_velocity: float = (0.0, 0.0, 0.0),
                show_flows: bool = False,
                start_baking: bool = False):
    '''Use selected objects in a fluid simulation 

    :param style: Fluid Style 
    :type style: typing.Union[int, str]
    :param initial_velocity: Initial Velocity, Initial velocity of the fluid 
    :type initial_velocity: float
    :param show_flows: Render Fluid Objects, Keep the fluid objects visible during rendering 
    :type show_flows: bool
    :param start_baking: Start Fluid Bake, Start baking the fluid immediately after creating the domain object 
    :type start_baking: bool
    '''

    pass


def quick_fur(density: typing.Union[int, str] = 'MEDIUM',
              view_percentage: int = 10,
              length: float = 0.1):
    '''Add fur setup to the selected objects 

    :param density: Fur Density 
    :type density: typing.Union[int, str]
    :param view_percentage: View % 
    :type view_percentage: int
    :param length: Length 
    :type length: float
    '''

    pass


def quick_smoke(style: typing.Union[int, str] = 'SMOKE',
                show_flows: bool = False):
    '''Use selected objects as smoke emitters 

    :param style: Smoke Style 
    :type style: typing.Union[int, str]
    :param show_flows: Render Smoke Objects, Keep the smoke objects visible during rendering 
    :type show_flows: bool
    '''

    pass


def randomize_transform(random_seed: int = 0,
                        use_delta: bool = False,
                        use_loc: bool = True,
                        loc: float = (0.0, 0.0, 0.0),
                        use_rot: bool = True,
                        rot: float = (0.0, 0.0, 0.0),
                        use_scale: bool = True,
                        scale_even: bool = False,
                        scale: float = (1.0, 1.0, 1.0)):
    '''Randomize objects loc/rot/scale 

    :param random_seed: Random Seed, Seed value for the random generator 
    :type random_seed: int
    :param use_delta: Transform Delta, Randomize delta transform values instead of regular transform 
    :type use_delta: bool
    :param use_loc: Randomize Location, Randomize the location values 
    :type use_loc: bool
    :param loc: Location, Maximum distance the objects can spread over each axis 
    :type loc: float
    :param use_rot: Randomize Rotation, Randomize the rotation values 
    :type use_rot: bool
    :param rot: Rotation, Maximum rotation over each axis 
    :type rot: float
    :param use_scale: Randomize Scale, Randomize the scale values 
    :type use_scale: bool
    :param scale_even: Scale Even, Use the same scale value for all axis 
    :type scale_even: bool
    :param scale: Scale, Maximum scale randomization over each axis 
    :type scale: float
    '''

    pass


def rotation_clear(clear_delta: bool = False):
    '''Clear the object’s rotation 

    :param clear_delta: Clear Delta, Clear delta rotation in addition to clearing the normal rotation transform 
    :type clear_delta: bool
    '''

    pass


def scale_clear(clear_delta: bool = False):
    '''Clear the object’s scale 

    :param clear_delta: Clear Delta, Clear delta scale in addition to clearing the normal scale transform 
    :type clear_delta: bool
    '''

    pass


def select_all(action: typing.Union[int, str] = 'TOGGLE'):
    '''Change selection of all visible objects in scene 

    :param action: Action, Selection action to executeTOGGLE Toggle, Toggle selection for all elements.SELECT Select, Select all elements.DESELECT Deselect, Deselect all elements.INVERT Invert, Invert selection of all elements. 
    :type action: typing.Union[int, str]
    '''

    pass


def select_by_type(extend: bool = False,
                   type: typing.Union[int, str] = 'MESH'):
    '''Select all visible objects that are of a type 

    :param extend: Extend, Extend selection instead of deselecting everything first 
    :type extend: bool
    :param type: Type 
    :type type: typing.Union[int, str]
    '''

    pass


def select_camera(extend: bool = False):
    '''Select the active camera 

    :param extend: Extend, Extend the selection 
    :type extend: bool
    '''

    pass


def select_grouped(extend: bool = False,
                   type: typing.Union[int, str] = 'CHILDREN_RECURSIVE'):
    '''Select all visible objects grouped by various properties 

    :param extend: Extend, Extend selection instead of deselecting everything first 
    :type extend: bool
    :param type: TypeCHILDREN_RECURSIVE Children.CHILDREN Immediate Children.PARENT Parent.SIBLINGS Siblings, Shared Parent.TYPE Type, Shared object type.COLLECTION Collection, Shared collection.HOOK Hook.PASS Pass, Render pass Index.COLOR Color, Object Color.KEYINGSET Keying Set, Objects included in active Keying Set.LIGHT_TYPE Light Type, Matching light types. 
    :type type: typing.Union[int, str]
    '''

    pass


def select_hierarchy(direction: typing.Union[int, str] = 'PARENT',
                     extend: bool = False):
    '''Select object relative to the active object’s position in the hierarchy 

    :param direction: Direction, Direction to select in the hierarchy 
    :type direction: typing.Union[int, str]
    :param extend: Extend, Extend the existing selection 
    :type extend: bool
    '''

    pass


def select_less():
    '''Deselect objects at the boundaries of parent/child relationships 

    '''

    pass


def select_linked(extend: bool = False,
                  type: typing.Union[int, str] = 'OBDATA'):
    '''Select all visible objects that are linked 

    :param extend: Extend, Extend selection instead of deselecting everything first 
    :type extend: bool
    :param type: Type 
    :type type: typing.Union[int, str]
    '''

    pass


def select_mirror(extend: bool = False):
    '''Select the Mirror objects of the selected object eg. L.sword -> R.sword 

    :param extend: Extend, Extend selection instead of deselecting everything first 
    :type extend: bool
    '''

    pass


def select_more():
    '''Select connected parent/child objects 

    '''

    pass


def select_pattern(pattern: str = "*",
                   case_sensitive: bool = False,
                   extend: bool = True):
    '''Select objects matching a naming pattern 

    :param pattern: Pattern, Name filter using ‘*’, ‘?’ and ‘[abc]’ unix style wildcards 
    :type pattern: str
    :param case_sensitive: Case Sensitive, Do a case sensitive compare 
    :type case_sensitive: bool
    :param extend: Extend, Extend the existing selection 
    :type extend: bool
    '''

    pass


def select_random(percent: float = 50.0,
                  seed: int = 0,
                  action: typing.Union[int, str] = 'SELECT'):
    '''Set select on random visible objects 

    :param percent: Percent, Percentage of objects to select randomly 
    :type percent: float
    :param seed: Random Seed, Seed for the random number generator 
    :type seed: int
    :param action: Action, Selection action to executeSELECT Select, Select all elements.DESELECT Deselect, Deselect all elements. 
    :type action: typing.Union[int, str]
    '''

    pass


def select_same_collection(collection: str = ""):
    '''Select object in the same collection 

    :param collection: Collection, Name of the collection to select 
    :type collection: str
    '''

    pass


def shade_flat():
    '''Render and display faces uniform, using Face Normals 

    '''

    pass


def shade_smooth():
    '''Render and display faces smooth, using interpolated Vertex Normals 

    '''

    pass


def shaderfx_add(type: typing.Union[int, str] = 'FX_BLUR'):
    '''Add a visual effect to the active object 

    :param type: TypeFX_BLUR Blur, Apply Gaussian Blur to object.FX_COLORIZE Colorize, Apply different tint effects.FX_FLIP Flip, Flip image.FX_GLOW Glow, Create a glow effect.FX_LIGHT Light, Simulate illumination.FX_PIXEL Pixelate, Pixelate image.FX_RIM Rim, Add a rim to the image.FX_SHADOW Shadow, Create a shadow effect.FX_SWIRL Swirl, Create a rotation distortion.FX_WAVE Wave Distortion, Apply sinusoidal deformation. 
    :type type: typing.Union[int, str]
    '''

    pass


def shaderfx_move_down(shaderfx: str = ""):
    '''Move shaderfx down in the stack 

    :param shaderfx: Shader, Name of the shaderfx to edit 
    :type shaderfx: str
    '''

    pass


def shaderfx_move_up(shaderfx: str = ""):
    '''Move shaderfx up in the stack 

    :param shaderfx: Shader, Name of the shaderfx to edit 
    :type shaderfx: str
    '''

    pass


def shaderfx_remove(shaderfx: str = ""):
    '''Remove a shaderfx from the active grease pencil object 

    :param shaderfx: Shader, Name of the shaderfx to edit 
    :type shaderfx: str
    '''

    pass


def shape_key_add(from_mix: bool = True):
    '''Add shape key to the object 

    :param from_mix: From Mix, Create the new shape key from the existing mix of keys 
    :type from_mix: bool
    '''

    pass


def shape_key_clear():
    '''Clear weights for all shape keys 

    '''

    pass


def shape_key_mirror(use_topology: bool = False):
    '''Mirror the current shape key along the local X axis 

    :param use_topology: Topology Mirror, Use topology based mirroring (for when both sides of mesh have matching, unique topology) 
    :type use_topology: bool
    '''

    pass


def shape_key_move(type: typing.Union[int, str] = 'TOP'):
    '''Move the active shape key up/down in the list 

    :param type: TypeTOP Top, Top of the list.UP Up.DOWN Down.BOTTOM Bottom, Bottom of the list. 
    :type type: typing.Union[int, str]
    '''

    pass


def shape_key_remove(all: bool = False):
    '''Remove shape key from the object 

    :param all: All, Remove all shape keys 
    :type all: bool
    '''

    pass


def shape_key_retime():
    '''Resets the timing for absolute shape keys 

    '''

    pass


def shape_key_transfer(mode: typing.Union[int, str] = 'OFFSET',
                       use_clamp: bool = False):
    '''Copy the active shape key of another selected object to this one 

    :param mode: Transformation Mode, Relative shape positions to the new shape methodOFFSET Offset, Apply the relative positional offset.RELATIVE_FACE Relative Face, Calculate relative position (using faces).RELATIVE_EDGE Relative Edge, Calculate relative position (using edges). 
    :type mode: typing.Union[int, str]
    :param use_clamp: Clamp Offset, Clamp the transformation to the distance each vertex moves in the original shape 
    :type use_clamp: bool
    '''

    pass


def skin_armature_create(modifier: str = ""):
    '''Create an armature that parallels the skin layout 

    :param modifier: Modifier, Name of the modifier to edit 
    :type modifier: str
    '''

    pass


def skin_loose_mark_clear(action: typing.Union[int, str] = 'MARK'):
    '''Mark/clear selected vertices as loose 

    :param action: ActionMARK Mark, Mark selected vertices as loose.CLEAR Clear, Set selected vertices as not loose. 
    :type action: typing.Union[int, str]
    '''

    pass


def skin_radii_equalize():
    '''Make skin radii of selected vertices equal on each axis 

    '''

    pass


def skin_root_mark():
    '''Mark selected vertices as roots 

    '''

    pass


def speaker_add(enter_editmode: bool = False,
                align: typing.Union[int, str] = 'WORLD',
                location: float = (0.0, 0.0, 0.0),
                rotation: float = (0.0, 0.0, 0.0)):
    '''Add a speaker object to the scene 

    :param enter_editmode: Enter Editmode, Enter editmode when adding this object 
    :type enter_editmode: bool
    :param align: Align, The alignment of the new objectWORLD World, Align the new object to the world.VIEW View, Align the new object to the view.CURSOR 3D Cursor, Use the 3D cursor orientation for the new object. 
    :type align: typing.Union[int, str]
    :param location: Location, Location for the newly added object 
    :type location: float
    :param rotation: Rotation, Rotation for the newly added object 
    :type rotation: float
    '''

    pass


def subdivision_set(level: int = 1, relative: bool = False):
    '''Sets a Subdivision Surface Level (1-5) 

    :param level: Level 
    :type level: int
    :param relative: Relative, Apply the subsurf level as an offset relative to the current level 
    :type relative: bool
    '''

    pass


def surfacedeform_bind(modifier: str = ""):
    '''Bind mesh to target in surface deform modifier 

    :param modifier: Modifier, Name of the modifier to edit 
    :type modifier: str
    '''

    pass


def text_add(radius: float = 1.0,
             enter_editmode: bool = False,
             align: typing.Union[int, str] = 'WORLD',
             location: float = (0.0, 0.0, 0.0),
             rotation: float = (0.0, 0.0, 0.0)):
    '''Add a text object to the scene 

    :param radius: Radius 
    :type radius: float
    :param enter_editmode: Enter Editmode, Enter editmode when adding this object 
    :type enter_editmode: bool
    :param align: Align, The alignment of the new objectWORLD World, Align the new object to the world.VIEW View, Align the new object to the view.CURSOR 3D Cursor, Use the 3D cursor orientation for the new object. 
    :type align: typing.Union[int, str]
    :param location: Location, Location for the newly added object 
    :type location: float
    :param rotation: Rotation, Rotation for the newly added object 
    :type rotation: float
    '''

    pass


def track_clear(type: typing.Union[int, str] = 'CLEAR'):
    '''Clear tracking constraint or flag from object 

    :param type: Type 
    :type type: typing.Union[int, str]
    '''

    pass


def track_set(type: typing.Union[int, str] = 'DAMPTRACK'):
    '''Make the object track another object, using various methods/constraints 

    :param type: Type 
    :type type: typing.Union[int, str]
    '''

    pass


def transform_apply(location: bool = True,
                    rotation: bool = True,
                    scale: bool = True,
                    properties: bool = True):
    '''Apply the object’s transformation to its data 

    :param location: Location 
    :type location: bool
    :param rotation: Rotation 
    :type rotation: bool
    :param scale: Scale 
    :type scale: bool
    :param properties: Apply Properties, Modify properties such as curve vertex radius, font size and bone envelope 
    :type properties: bool
    '''

    pass


def transform_axis_target():
    '''Interactively point cameras and lights to a location (Ctrl translates) 

    '''

    pass


def transforms_to_deltas(mode: typing.Union[int, str] = 'ALL',
                         reset_values: bool = True):
    '''Convert normal object transforms to delta transforms, any existing delta transforms will be included as well 

    :param mode: Mode, Which transforms to transferALL All Transforms, Transfer location, rotation, and scale transforms.LOC Location, Transfer location transforms only.ROT Rotation, Transfer rotation transforms only.SCALE Scale, Transfer scale transforms only. 
    :type mode: typing.Union[int, str]
    :param reset_values: Reset Values, Clear transform values after transferring to deltas 
    :type reset_values: bool
    '''

    pass


def unlink_data():
    '''Undocumented contribute <https://developer.blender.org/T51061> 

    '''

    pass


def vertex_group_add():
    '''Add a new vertex group to the active object 

    '''

    pass


def vertex_group_assign():
    '''Assign the selected vertices to the active vertex group 

    '''

    pass


def vertex_group_assign_new():
    '''Assign the selected vertices to a new vertex group 

    '''

    pass


def vertex_group_clean(group_select_mode: typing.Union[int, str] = '',
                       limit: float = 0.0,
                       keep_single: bool = False):
    '''Remove vertex group assignments which are not required 

    :param group_select_mode: Subset, Define which subset of Groups shall be used 
    :type group_select_mode: typing.Union[int, str]
    :param limit: Limit, Remove vertices which weight is below or equal to this limit 
    :type limit: float
    :param keep_single: Keep Single, Keep verts assigned to at least one group when cleaning 
    :type keep_single: bool
    '''

    pass


def vertex_group_copy():
    '''Make a copy of the active vertex group 

    '''

    pass


def vertex_group_copy_to_linked():
    '''Replace vertex groups of all users of the same geometry data by vertex groups of active object 

    '''

    pass


def vertex_group_copy_to_selected():
    '''Replace vertex groups of selected objects by vertex groups of active object 

    '''

    pass


def vertex_group_deselect():
    '''Deselect all selected vertices assigned to the active vertex group 

    '''

    pass


def vertex_group_fix(dist: float = 0.0,
                     strength: float = 1.0,
                     accuracy: float = 1.0):
    '''Modify the position of selected vertices by changing only their respective groups’ weights (this tool may be slow for many vertices) 

    :param dist: Distance, The distance to move to 
    :type dist: float
    :param strength: Strength, The distance moved can be changed by this multiplier 
    :type strength: float
    :param accuracy: Change Sensitivity, Change the amount weights are altered with each iteration: lower values are slower 
    :type accuracy: float
    '''

    pass


def vertex_group_invert(group_select_mode: typing.Union[int, str] = '',
                        auto_assign: bool = True,
                        auto_remove: bool = True):
    '''Invert active vertex group’s weights 

    :param group_select_mode: Subset, Define which subset of Groups shall be used 
    :type group_select_mode: typing.Union[int, str]
    :param auto_assign: Add Weights, Add verts from groups that have zero weight before inverting 
    :type auto_assign: bool
    :param auto_remove: Remove Weights, Remove verts from groups that have zero weight after inverting 
    :type auto_remove: bool
    '''

    pass


def vertex_group_levels(group_select_mode: typing.Union[int, str] = '',
                        offset: float = 0.0,
                        gain: float = 1.0):
    '''Add some offset and multiply with some gain the weights of the active vertex group 

    :param group_select_mode: Subset, Define which subset of Groups shall be used 
    :type group_select_mode: typing.Union[int, str]
    :param offset: Offset, Value to add to weights 
    :type offset: float
    :param gain: Gain, Value to multiply weights by 
    :type gain: float
    '''

    pass


def vertex_group_limit_total(group_select_mode: typing.Union[int, str] = '',
                             limit: int = 4):
    '''Limit deform weights associated with a vertex to a specified number by removing lowest weights 

    :param group_select_mode: Subset, Define which subset of Groups shall be used 
    :type group_select_mode: typing.Union[int, str]
    :param limit: Limit, Maximum number of deform weights 
    :type limit: int
    '''

    pass


def vertex_group_lock(action: typing.Union[int, str] = 'TOGGLE'):
    '''Change the lock state of all vertex groups of active object 

    :param action: Action, Lock action to execute on vertex groupsTOGGLE Toggle, Unlock all vertex groups if there is at least one locked group, lock all in other case.LOCK Lock, Lock all vertex groups.UNLOCK Unlock, Unlock all vertex groups.INVERT Invert, Invert the lock state of all vertex groups. 
    :type action: typing.Union[int, str]
    '''

    pass


def vertex_group_mirror(mirror_weights: bool = True,
                        flip_group_names: bool = True,
                        all_groups: bool = False,
                        use_topology: bool = False):
    '''Mirror vertex group, flip weights and/or names, editing only selected vertices, flipping when both sides are selected otherwise copy from unselected 

    :param mirror_weights: Mirror Weights, Mirror weights 
    :type mirror_weights: bool
    :param flip_group_names: Flip Group Names, Flip vertex group names 
    :type flip_group_names: bool
    :param all_groups: All Groups, Mirror all vertex groups weights 
    :type all_groups: bool
    :param use_topology: Topology Mirror, Use topology based mirroring (for when both sides of mesh have matching, unique topology) 
    :type use_topology: bool
    '''

    pass


def vertex_group_move(direction: typing.Union[int, str] = 'UP'):
    '''Move the active vertex group up/down in the list 

    :param direction: Direction, Direction to move the active vertex group towards 
    :type direction: typing.Union[int, str]
    '''

    pass


def vertex_group_normalize():
    '''Normalize weights of the active vertex group, so that the highest ones are now 1.0 

    '''

    pass


def vertex_group_normalize_all(group_select_mode: typing.Union[int, str] = '',
                               lock_active: bool = True):
    '''Normalize all weights of all vertex groups, so that for each vertex, the sum of all weights is 1.0 

    :param group_select_mode: Subset, Define which subset of Groups shall be used 
    :type group_select_mode: typing.Union[int, str]
    :param lock_active: Lock Active, Keep the values of the active group while normalizing others 
    :type lock_active: bool
    '''

    pass


def vertex_group_quantize(group_select_mode: typing.Union[int, str] = '',
                          steps: int = 4):
    '''Set weights to a fixed number of steps 

    :param group_select_mode: Subset, Define which subset of Groups shall be used 
    :type group_select_mode: typing.Union[int, str]
    :param steps: Steps, Number of steps between 0 and 1 
    :type steps: int
    '''

    pass


def vertex_group_remove(all: bool = False, all_unlocked: bool = False):
    '''Delete the active or all vertex groups from the active object 

    :param all: All, Remove all vertex groups 
    :type all: bool
    :param all_unlocked: All Unlocked, Remove all unlocked vertex groups 
    :type all_unlocked: bool
    '''

    pass


def vertex_group_remove_from(use_all_groups: bool = False,
                             use_all_verts: bool = False):
    '''Remove the selected vertices from active or all vertex group(s) 

    :param use_all_groups: All Groups, Remove from all groups 
    :type use_all_groups: bool
    :param use_all_verts: All Verts, Clear the active group 
    :type use_all_verts: bool
    '''

    pass


def vertex_group_select():
    '''Select all the vertices assigned to the active vertex group 

    '''

    pass


def vertex_group_set_active(group: typing.Union[int, str] = ''):
    '''Set the active vertex group 

    :param group: Group, Vertex group to set as active 
    :type group: typing.Union[int, str]
    '''

    pass


def vertex_group_smooth(group_select_mode: typing.Union[int, str] = '',
                        factor: float = 0.5,
                        repeat: int = 1,
                        expand: float = 0.0):
    '''Smooth weights for selected vertices 

    :param group_select_mode: Subset, Define which subset of Groups shall be used 
    :type group_select_mode: typing.Union[int, str]
    :param factor: Factor 
    :type factor: float
    :param repeat: Iterations 
    :type repeat: int
    :param expand: Expand/Contract, Expand/contract weights 
    :type expand: float
    '''

    pass


def vertex_group_sort(sort_type: typing.Union[int, str] = 'NAME'):
    '''Sort vertex groups 

    :param sort_type: Sort type, Sort type 
    :type sort_type: typing.Union[int, str]
    '''

    pass


def vertex_parent_set():
    '''Parent selected objects to the selected vertices 

    '''

    pass


def vertex_weight_copy():
    '''Copy weights from active to selected 

    '''

    pass


def vertex_weight_delete(weight_group: int = -1):
    '''Delete this weight from the vertex (disabled if vertex group is locked) 

    :param weight_group: Weight Index, Index of source weight in active vertex group 
    :type weight_group: int
    '''

    pass


def vertex_weight_normalize_active_vertex():
    '''Normalize active vertex’s weights 

    '''

    pass


def vertex_weight_paste(weight_group: int = -1):
    '''Copy this group’s weight to other selected verts (disabled if vertex group is locked) 

    :param weight_group: Weight Index, Index of source weight in active vertex group 
    :type weight_group: int
    '''

    pass


def vertex_weight_set_active(weight_group: int = -1):
    '''Set as active vertex group 

    :param weight_group: Weight Index, Index of source weight in active vertex group 
    :type weight_group: int
    '''

    pass


def visual_transform_apply():
    '''Apply the object’s visual transformation to its data 

    '''

    pass



import sys
import typing


def copy():
    '''Copy the material settings and nodes 

    '''

    pass


def new():
    '''Add a new material 

    '''

    pass


def paste():
    '''Paste the material settings and nodes 

    '''

    pass



import sys
import typing
import bpy
import mathutils


def location_3d_to_region_2d(region: 'bpy.types.Region',
                             rv3d: 'bpy.types.RegionView3D',
                             coord,
                             default=None) -> 'mathutils.Vector':
    '''Return the region relative 2d location of a 3d position. 

    :param region: region of the 3D viewport, typically bpy.context.region. 
    :type region: 'bpy.types.Region'
    :param rv3d: 3D region data, typically bpy.context.space_data.region_3d. 
    :type rv3d: 'bpy.types.RegionView3D'
    :param coord: 3d worldspace location. 
    :param default: Return this value if coord is behind the origin of a perspective view. 
    :return:  2d location 
    '''

    pass


def region_2d_to_location_3d(region: 'bpy.types.Region',
                             rv3d: 'bpy.types.RegionView3D', coord,
                             depth_location) -> 'mathutils.Vector':
    '''Return a 3d location from the region relative 2d coords, aligned with depth_location. 

    :param region: region of the 3D viewport, typically bpy.context.region. 
    :type region: 'bpy.types.Region'
    :param rv3d: 3D region data, typically bpy.context.space_data.region_3d. 
    :type rv3d: 'bpy.types.RegionView3D'
    :param coord: 2d coordinates relative to the region; (event.mouse_region_x, event.mouse_region_y) for example. 
    :param depth_location: the returned vectors depth is aligned with this since there is no defined depth with a 2d region input. 
    :return:  normalized 3d vector. 
    '''

    pass


def region_2d_to_origin_3d(region: 'bpy.types.Region',
                           rv3d: 'bpy.types.RegionView3D',
                           coord,
                           clamp: float = None) -> 'mathutils.Vector':
    '''Return the 3d view origin from the region relative 2d coords. 

    :param region: region of the 3D viewport, typically bpy.context.region. 
    :type region: 'bpy.types.Region'
    :param rv3d: 3D region data, typically bpy.context.space_data.region_3d. 
    :type rv3d: 'bpy.types.RegionView3D'
    :param coord: 2d coordinates relative to the region; (event.mouse_region_x, event.mouse_region_y) for example. 
    :param clamp: Clamp the maximum far-clip value used. (negative value will move the offset away from the view_location) 
    :type clamp: float
    :return:  The origin of the viewpoint in 3d space. 
    '''

    pass


def region_2d_to_vector_3d(region: 'bpy.types.Region',
                           rv3d: 'bpy.types.RegionView3D',
                           coord) -> 'mathutils.Vector':
    '''Return a direction vector from the viewport at the specific 2d region coordinate. 

    :param region: region of the 3D viewport, typically bpy.context.region. 
    :type region: 'bpy.types.Region'
    :param rv3d: 3D region data, typically bpy.context.space_data.region_3d. 
    :type rv3d: 'bpy.types.RegionView3D'
    :param coord: 2d coordinates relative to the region: (event.mouse_region_x, event.mouse_region_y) for example. 
    :return:  normalized 3d vector. 
    '''

    pass



import sys
import typing
import bpy


class ExportHelper:
    def check(self, _context):
        '''

        '''
        pass

    def invoke(self, context, _event):
        '''

        '''
        pass


class ImportHelper:
    def check(self, _context):
        '''

        '''
        pass

    def invoke(self, context, _event):
        '''

        '''
        pass


path_reference_mode = None
'''constant value (<built-in function EnumProperty>, {‘name: ‘Path Mode, ‘description: ‘Method used to reference paths, ‘items: ((‘AUTO, ‘Auto, ‘Use Relative paths with subdirectories only), (‘ABSOLUTE, ‘Absolute, ‘Always write absolute paths), (‘RELATIVE, ‘Relative, ‘Always write relative paths (where possible)), (‘MATCH, ‘Match, ‘Match Absolute/Relative setting with input path), (‘STRIP, ‘Strip Path, ‘Filename only), (‘COPY, ‘Copy, ‘Copy the file to the destination path (or subdirectory))), ‘default: ‘AUTO, ‘attr: ‘path_mode}) '''


def axis_conversion(from_forward='Y', from_up='Z', to_forward='Y', to_up='Z'):
    '''Each argument us an axis in [‘X’, ‘Y’, ‘Z’, ‘-X’, ‘-Y’, ‘-Z’] where the first 2 are a source and the second 2 are the target. 

    '''

    pass


def axis_conversion_ensure(operator: 'bpy.types.Operator', forward_attr: str,
                           up_attr: str) -> bool:
    '''Function to ensure an operator has valid axis conversion settings, intended to be used from bpy.types.Operator.check. 

    :param operator: the operator to access axis attributes from. 
    :type operator: 'bpy.types.Operator'
    :param forward_attr: attribute storing the forward axis 
    :type forward_attr: str
    :param up_attr: attribute storing the up axis 
    :type up_attr: str
    :return:  True if the value was modified. 
    '''

    pass


def create_derived_objects(scene, ob):
    '''

    '''

    pass


def free_derived_objects(ob):
    '''

    '''

    pass


def orientation_helper(axis_forward='Y', axis_up='Z'):
    '''A decorator for import/export classes, generating properties needed by the axis conversion system and IO helpers, with specified default values (axes). 

    '''

    pass


def path_reference(filepath: str,
                   base_src: str,
                   base_dst: str,
                   mode: str = 'AUTO',
                   copy_subdir: str = '',
                   copy_set: set = None,
                   library: 'bpy.types.Library' = None) -> str:
    '''Return a filepath relative to a destination directory, for use with exporters. 

    :param filepath: the file path to return, supporting blenders relative ‘//’ prefix. 
    :type filepath: str
    :param base_src: the directory the filepath is relative too (normally the blend file). 
    :type base_src: str
    :param base_dst: the directory the filepath will be referenced from (normally the export path). 
    :type base_dst: str
    :param mode: the method used get the path in [‘AUTO’, ‘ABSOLUTE’, ‘RELATIVE’, ‘MATCH’, ‘STRIP’, ‘COPY’] 
    :type mode: str
    :param copy_subdir: the subdirectory of base_dst to use when mode=’COPY’. 
    :type copy_subdir: str
    :param copy_set: collect from/to pairs when mode=’COPY’, pass to path_reference_copy when exporting is done. 
    :type copy_set: set
    :param library: The library this path is relative to. 
    :type library: 'bpy.types.Library'
    :return:  the new filepath. 
    '''

    pass


def path_reference_copy(copy_set: set, report='print'):
    '''Execute copying files of path_reference 

    :param copy_set: set of (from, to) pairs to copy. 
    :type copy_set: set
    :param report: function used for reporting warnings, takes a string argument. 
    '''

    pass


def unique_name(key: 'bpy.context.object',
                name: str,
                name_dict: dict,
                name_max=-1,
                clean_func=None,
                sep: str = '.'):
    '''Helper function for storing unique names which may have special characters stripped and restricted to a maximum length. 

    :param key: unique item this name belongs to, name_dict[key] will be reused when available. This can be the object, mesh, material, etc instance its self. 
    :type key: 'bpy.context.object'
    :param name: The name used to create a unique value in name_dict. 
    :type name: str
    :param name_dict: This is used to cache namespace to ensure no collisions occur, this should be an empty dict initially and only modified by this function. 
    :type name_dict: dict
    :param clean_func: Function to call on name before creating a unique value. 
    :param sep: Separator to use when between the name and a number when a duplicate name is found. 
    :type sep: str
    '''

    pass


def unpack_face_list(list_of_tuples):
    '''

    '''

    pass


def unpack_list(list_of_tuples):
    '''

    '''

    pass



import sys
import typing
import bpy


def clean(threshold: float = 0.001, channels: bool = False):
    '''Simplify F-Curves by removing closely spaced keyframes 

    :param threshold: Threshold 
    :type threshold: float
    :param channels: Channels 
    :type channels: bool
    '''

    pass


def clickselect(extend: bool = False,
                deselect_all: bool = False,
                column: bool = False,
                channel: bool = False):
    '''Select keyframes by clicking on them 

    :param extend: Extend Select, Toggle keyframe selection instead of leaving newly selected keyframes only 
    :type extend: bool
    :param deselect_all: Deselect On Nothing, Deselect all when nothing under the cursor 
    :type deselect_all: bool
    :param column: Column Select, Select all keyframes that occur on the same frame as the one under the mouse 
    :type column: bool
    :param channel: Only Channel, Select all the keyframes in the channel under the mouse 
    :type channel: bool
    '''

    pass


def copy():
    '''Copy selected keyframes to the copy/paste buffer 

    '''

    pass


def delete():
    '''Remove all selected keyframes 

    '''

    pass


def duplicate():
    '''Make a copy of all selected keyframes 

    '''

    pass


def duplicate_move(ACTION_OT_duplicate=None, TRANSFORM_OT_transform=None):
    '''Make a copy of all selected keyframes and move them 

    :param ACTION_OT_duplicate: Duplicate Keyframes, Make a copy of all selected keyframes 
    :param TRANSFORM_OT_transform: Transform, Transform selected items by mode type 
    '''

    pass


def extrapolation_type(type: typing.Union[int, str] = 'CONSTANT'):
    '''Set extrapolation mode for selected F-Curves 

    :param type: TypeCONSTANT Constant Extrapolation, Values on endpoint keyframes are held.LINEAR Linear Extrapolation, Straight-line slope of end segments are extended past the endpoint keyframes.MAKE_CYCLIC Make Cyclic (F-Modifier), Add Cycles F-Modifier if one doesn’t exist already.CLEAR_CYCLIC Clear Cyclic (F-Modifier), Remove Cycles F-Modifier if not needed anymore. 
    :type type: typing.Union[int, str]
    '''

    pass


def frame_jump():
    '''Set the current frame to the average frame value of selected keyframes 

    '''

    pass


def handle_type(type: typing.Union[int, str] = 'FREE'):
    '''Set type of handle for selected keyframes 

    :param type: TypeFREE Free, Completely independent manually set handle.ALIGNED Aligned, Manually set handle with rotation locked together with its pair.VECTOR Vector, Automatic handles that create straight lines.AUTO Automatic, Automatic handles that create smooth curves.AUTO_CLAMPED Auto Clamped, Automatic handles that create smooth curves which only change direction at keyframes. 
    :type type: typing.Union[int, str]
    '''

    pass


def interpolation_type(type: typing.Union[int, str] = 'CONSTANT'):
    '''Set interpolation mode for the F-Curve segments starting from the selected keyframes 

    :param type: TypeCONSTANT Constant, No interpolation, value of A gets held until B is encountered.LINEAR Linear, Straight-line interpolation between A and B (i.e. no ease in/out).BEZIER Bezier, Smooth interpolation between A and B, with some control over curve shape.SINE Sinusoidal, Sinusoidal easing (weakest, almost linear but with a slight curvature).QUAD Quadratic, Quadratic easing.CUBIC Cubic, Cubic easing.QUART Quartic, Quartic easing.QUINT Quintic, Quintic easing.EXPO Exponential, Exponential easing (dramatic).CIRC Circular, Circular easing (strongest and most dynamic).BACK Back, Cubic easing with overshoot and settle.BOUNCE Bounce, Exponentially decaying parabolic bounce, like when objects collide.ELASTIC Elastic, Exponentially decaying sine wave, like an elastic band. 
    :type type: typing.Union[int, str]
    '''

    pass


def keyframe_insert(type: typing.Union[int, str] = 'ALL'):
    '''Insert keyframes for the specified channels 

    :param type: Type 
    :type type: typing.Union[int, str]
    '''

    pass


def keyframe_type(type: typing.Union[int, str] = 'KEYFRAME'):
    '''Set type of keyframe for the selected keyframes 

    :param type: TypeKEYFRAME Keyframe, Normal keyframe - e.g. for key poses.BREAKDOWN Breakdown, A breakdown pose - e.g. for transitions between key poses.MOVING_HOLD Moving Hold, A keyframe that is part of a moving hold.EXTREME Extreme, An ‘extreme’ pose, or some other purpose as needed.JITTER Jitter, A filler or baked keyframe for keying on ones, or some other purpose as needed. 
    :type type: typing.Union[int, str]
    '''

    pass


def layer_next():
    '''Switch to editing action in animation layer above the current action in the NLA Stack 

    '''

    pass


def layer_prev():
    '''Switch to editing action in animation layer below the current action in the NLA Stack 

    '''

    pass


def markers_make_local():
    '''Move selected scene markers to the active Action as local ‘pose’ markers 

    '''

    pass


def mirror(type: typing.Union[int, str] = 'CFRA'):
    '''Flip selected keyframes over the selected mirror line 

    :param type: TypeCFRA By Times over Current frame, Flip times of selected keyframes using the current frame as the mirror line.XAXIS By Values over Value=0, Flip values of selected keyframes (i.e. negative values become positive, and vice versa).MARKER By Times over First Selected Marker, Flip times of selected keyframes using the first selected marker as the reference point. 
    :type type: typing.Union[int, str]
    '''

    pass


def new():
    '''Create new action 

    '''

    pass


def paste(offset: typing.Union[int, str] = 'START',
          merge: typing.Union[int, str] = 'MIX',
          flipped: bool = False):
    '''Paste keyframes from copy/paste buffer for the selected channels, starting on the current frame 

    :param offset: Offset, Paste time offset of keysSTART Frame Start, Paste keys starting at current frame.END Frame End, Paste keys ending at current frame.RELATIVE Frame Relative, Paste keys relative to the current frame when copying.NONE No Offset, Paste keys from original time. 
    :type offset: typing.Union[int, str]
    :param merge: Type, Method of merging pasted keys and existingMIX Mix, Overlay existing with new keys.OVER_ALL Overwrite All, Replace all keys.OVER_RANGE Overwrite Range, Overwrite keys in pasted range.OVER_RANGE_ALL Overwrite Entire Range, Overwrite keys in pasted range, using the range of all copied keys. 
    :type merge: typing.Union[int, str]
    :param flipped: Flipped, Paste keyframes from mirrored bones if they exist 
    :type flipped: bool
    '''

    pass


def previewrange_set():
    '''Set Preview Range based on extents of selected Keyframes 

    '''

    pass


def push_down():
    '''Push action down on to the NLA stack as a new strip 

    '''

    pass


def sample():
    '''Add keyframes on every frame between the selected keyframes 

    '''

    pass


def select_all(action: typing.Union[int, str] = 'TOGGLE'):
    '''Toggle selection of all keyframes 

    :param action: Action, Selection action to executeTOGGLE Toggle, Toggle selection for all elements.SELECT Select, Select all elements.DESELECT Deselect, Deselect all elements.INVERT Invert, Invert selection of all elements. 
    :type action: typing.Union[int, str]
    '''

    pass


def select_box(axis_range: bool = False,
               xmin: int = 0,
               xmax: int = 0,
               ymin: int = 0,
               ymax: int = 0,
               wait_for_input: bool = True,
               mode: typing.Union[int, str] = 'SET',
               tweak: bool = False):
    '''Select all keyframes within the specified region 

    :param axis_range: Axis Range 
    :type axis_range: bool
    :param xmin: X Min 
    :type xmin: int
    :param xmax: X Max 
    :type xmax: int
    :param ymin: Y Min 
    :type ymin: int
    :param ymax: Y Max 
    :type ymax: int
    :param wait_for_input: Wait for Input 
    :type wait_for_input: bool
    :param mode: ModeSET Set, Set a new selection.ADD Extend, Extend existing selection.SUB Subtract, Subtract existing selection. 
    :type mode: typing.Union[int, str]
    :param tweak: Tweak, Operator has been activated using a tweak event 
    :type tweak: bool
    '''

    pass


def select_circle(x: int = 0,
                  y: int = 0,
                  radius: int = 25,
                  wait_for_input: bool = True,
                  mode: typing.Union[int, str] = 'SET'):
    '''Select keyframe points using circle selection 

    :param x: X 
    :type x: int
    :param y: Y 
    :type y: int
    :param radius: Radius 
    :type radius: int
    :param wait_for_input: Wait for Input 
    :type wait_for_input: bool
    :param mode: ModeSET Set, Set a new selection.ADD Extend, Extend existing selection.SUB Subtract, Subtract existing selection. 
    :type mode: typing.Union[int, str]
    '''

    pass


def select_column(mode: typing.Union[int, str] = 'KEYS'):
    '''Select all keyframes on the specified frame(s) 

    :param mode: Mode 
    :type mode: typing.Union[int, str]
    '''

    pass


def select_lasso(path: typing.List['bpy.types.OperatorMousePath'] = None,
                 mode: typing.Union[int, str] = 'SET'):
    '''Select keyframe points using lasso selection 

    :param path: Path 
    :type path: typing.List['bpy.types.OperatorMousePath']
    :param mode: ModeSET Set, Set a new selection.ADD Extend, Extend existing selection.SUB Subtract, Subtract existing selection. 
    :type mode: typing.Union[int, str]
    '''

    pass


def select_leftright(mode: typing.Union[int, str] = 'CHECK',
                     extend: bool = False):
    '''Select keyframes to the left or the right of the current frame 

    :param mode: Mode 
    :type mode: typing.Union[int, str]
    :param extend: Extend Select 
    :type extend: bool
    '''

    pass


def select_less():
    '''Deselect keyframes on ends of selection islands 

    '''

    pass


def select_linked():
    '''Select keyframes occurring in the same F-Curves as selected ones 

    '''

    pass


def select_more():
    '''Select keyframes beside already selected ones 

    '''

    pass


def snap(type: typing.Union[int, str] = 'CFRA'):
    '''Snap selected keyframes to the times specified 

    :param type: TypeCFRA Current frame, Snap selected keyframes to the current frame.NEAREST_FRAME Nearest Frame, Snap selected keyframes to the nearest (whole) frame (use to fix accidental sub-frame offsets).NEAREST_SECOND Nearest Second, Snap selected keyframes to the nearest second.NEAREST_MARKER Nearest Marker, Snap selected keyframes to the nearest marker. 
    :type type: typing.Union[int, str]
    '''

    pass


def stash(create_new: bool = True):
    '''Store this action in the NLA stack as a non-contributing strip for later use 

    :param create_new: Create New Action, Create a new action once the existing one has been safely stored 
    :type create_new: bool
    '''

    pass


def stash_and_create():
    '''Store this action in the NLA stack as a non-contributing strip for later use, and create a new action 

    '''

    pass


def unlink(force_delete: bool = False):
    '''Unlink this action from the active action slot (and/or exit Tweak Mode) 

    :param force_delete: Force Delete, Clear Fake User and remove copy stashed in this data-block’s NLA stack 
    :type force_delete: bool
    '''

    pass


def view_all():
    '''Reset viewable area to show full keyframe range 

    '''

    pass


def view_frame():
    '''Reset viewable area to show range around current frame 

    '''

    pass


def view_selected():
    '''Reset viewable area to show selected keyframes range 

    '''

    pass



import sys
import typing
import bpy
import mathutils


class GPUBatch:
    '''Reusable container for drawable geometry. '''

    def draw(self, program: 'GPUShader' = None):
        '''Run the drawing program with the parameters assigned to the batch. 

        :param program: Program that performs the drawing operations. If None is passed, the last program setted to this batch will run. 
        :type program: 'GPUShader'
        '''
        pass

    def program_set(self, program: 'GPUShader'):
        '''Assign a shader to this batch that will be used for drawing when not overwritten later. Note: This method has to be called in the draw context that the batch will be drawn in. This function does not need to be called when you always set the shader when calling batch.draw. 

        :param program: The program/shader the batch will use in future draw calls. 
        :type program: 'GPUShader'
        '''
        pass

    def vertbuf_add(self, buf: 'GPUVertBuf'):
        '''Add another vertex buffer to the Batch. It is not possible to add more vertices to the batch using this method. Instead it can be used to add more attributes to the existing vertices. A good use case would be when you have a separate vertex buffer for vertex positions and vertex normals. Current a batch can have at most 4 vertex buffers. 

        :param buf: The vertex buffer that will be added to the batch. 
        :type buf: 'GPUVertBuf'
        '''
        pass


class GPUIndexBuf:
    '''Contains an index buffer. '''

    pass


class GPUOffScreen:
    '''This object gives access to off screen buffers. '''

    color_texture: int = None
    '''OpenGL bindcode for the color texture. 

    :type: int
    '''

    height: int = None
    '''Height of the texture. 

    :type: int
    '''

    width: int = None
    '''Width of the texture. 

    :type: int
    '''

    def bind(self, save: bool = True):
        '''Bind the offscreen object. To make sure that the offscreen gets unbind whether an exception occurs or not, pack it into a with statement. 

        :param save: Save the current OpenGL state, so that it can be restored when unbinding. 
        :type save: bool
        '''
        pass

    def draw_view3d(
            self, scene: 'bpy.types.Scene', view3d: 'bpy.types.SpaceView3D',
            region: 'bpy.types.Region', view_matrix: 'mathutils.Matrix',
            projection_matrix: 'mathutils.Matrix'):
        '''Draw the 3d viewport in the offscreen object. 

        :param scene: Scene to draw. 
        :type scene: 'bpy.types.Scene'
        :param view_layer: View layer to draw. 
        :type view_layer: 'bpy.types.ViewLayer'
        :param view3d: 3D View to get the drawing settings from. 
        :type view3d: 'bpy.types.SpaceView3D'
        :param region: Region of the 3D View (required as temporary draw target). 
        :type region: 'bpy.types.Region'
        :param view_matrix: View Matrix (e.g. camera.matrix_world.inverted()). 
        :type view_matrix: 'mathutils.Matrix'
        :param projection_matrix: Projection Matrix (e.g. camera.calc_matrix_camera(...)). 
        :type projection_matrix: 'mathutils.Matrix'
        '''
        pass

    def free(self, ):
        '''Free the offscreen object. The framebuffer, texture and render objects will no longer be accessible. 

        '''
        pass

    def unbind(self, restore: bool = True):
        '''Unbind the offscreen object. 

        :param restore: Restore the OpenGL state, can only be used when the state has been saved before. 
        :type restore: bool
        '''
        pass


class GPUShader:
    '''To debug shaders, use the –debug-gpu-shaders command line option to see full GLSL shader compilation and linking errors. '''

    program: int = None
    '''The name of the program object for use by the OpenGL API (read-only). 

    :type: int
    '''

    def attr_from_name(self, name: str) -> int:
        '''Get attribute location by name. 

        :param name: The name of the attribute variable whose location is to be queried. 
        :type name: str
        :rtype: int
        :return:  The location of an attribute variable. 
        '''
        pass

    def bind(self, ):
        '''Bind the shader object. Required to be able to change uniforms of this shader. 

        '''
        pass

    def calc_format(self, ) -> 'GPUVertFormat':
        '''Build a new format based on the attributes of the shader. 

        :rtype: 'GPUVertFormat'
        :return:  vertex attribute format for the shader 
        '''
        pass

    def uniform_block_from_name(self, name: str) -> int:
        '''Get uniform block location by name. 

        :param name: Name of the uniform block variable whose location is to be queried. 
        :type name: str
        :rtype: int
        :return:  The location of the uniform block variable. 
        '''
        pass

    def uniform_bool(self, name: str, seq: list):
        '''Specify the value of a uniform variable for the current program object. 

        :param name: Name of the uniform variable whose value is to be changed. 
        :type name: str
        :param seq: Value that will be used to update the specified uniform variable. 
        :type seq: list
        '''
        pass

    def uniform_float(self, name: str, value: list):
        '''Specify the value of a uniform variable for the current program object. 

        :param name: Name of the uniform variable whose value is to be changed. 
        :type name: str
        :param value: Value that will be used to update the specified uniform variable. 
        :type value: list
        '''
        pass

    def uniform_from_name(self, name: str) -> int:
        '''Get uniform location by name. 

        :param name: Name of the uniform variable whose location is to be queried. 
        :type name: str
        :rtype: int
        :return:  Location of the uniform variable. 
        '''
        pass

    def uniform_int(self, name: str, seq: list):
        '''Specify the value of a uniform variable for the current program object. 

        :param name: name of the uniform variable whose value is to be changed. 
        :type name: str
        :param seq: Value that will be used to update the specified uniform variable. 
        :type seq: list
        '''
        pass

    def uniform_vector_float(self, location: int, buffer: list, length: int,
                             count: int):
        '''Set the buffer to fill the uniform. 

        :param location: Location of the uniform variable to be modified. 
        :type location: int
        :param buffer: The data that should be set. Can support the buffer protocol. 
        :type buffer: list
        :param length: Size of the uniform data type:1: float2: vec2 or float[2]3: vec3 or float[3]4: vec4 or float[4]9: mat316: mat4 
        :type length: int
        :param count: Specifies the number of elements, vector or matrices that are to be modified. 
        :type count: int
        '''
        pass

    def uniform_vector_int(self, location, buffer, length, count):
        '''See GPUShader.uniform_vector_float(…) description. 

        '''
        pass


class GPUVertBuf:
    '''Contains a VBO. '''

    def attr_fill(self, id: typing.Union[int, str], data: list):
        '''Insert data into the buffer for a single attribute. 

        :param id: Either the name or the id of the attribute. 
        :type id: typing.Union[int, str]
        :param data: Sequence of data that should be stored in the buffer 
        :type data: list
        '''
        pass


class GPUVertFormat:
    '''This object contains information about the structure of a vertex buffer. '''

    def attr_add(self, id: str, comp_type: str, len: int, fetch_mode: str):
        '''Add a new attribute to the format. 

        :param id: Name the attribute. Often position, normal, … 
        :type id: str
        :param comp_type: The data type that will be used store the value in memory. Possible values are I8, U8, I16, U16, I32, U32, F32 and I10. 
        :type comp_type: str
        :param len: How many individual values the attribute consists of (e.g. 2 for uv coordinates). 
        :type len: int
        :param fetch_mode: How values from memory will be converted when used in the shader. This is mainly useful for memory optimizations when you want to store values with reduced precision. E.g. you can store a float in only 1 byte but it will be converted to a normal 4 byte float when used. Possible values are FLOAT, INT, INT_TO_FLOAT_UNIT and INT_TO_FLOAT. 
        :type fetch_mode: str
        '''
        pass



import sys
import typing


def primitive_nurbs_surface_circle_add(radius: float = 1.0,
                                       enter_editmode: bool = False,
                                       align: typing.Union[int, str] = 'WORLD',
                                       location: float = (0.0, 0.0, 0.0),
                                       rotation: float = (0.0, 0.0, 0.0)):
    '''Construct a Nurbs surface Circle 

    :param radius: Radius 
    :type radius: float
    :param enter_editmode: Enter Editmode, Enter editmode when adding this object 
    :type enter_editmode: bool
    :param align: Align, The alignment of the new objectWORLD World, Align the new object to the world.VIEW View, Align the new object to the view.CURSOR 3D Cursor, Use the 3D cursor orientation for the new object. 
    :type align: typing.Union[int, str]
    :param location: Location, Location for the newly added object 
    :type location: float
    :param rotation: Rotation, Rotation for the newly added object 
    :type rotation: float
    '''

    pass


def primitive_nurbs_surface_curve_add(radius: float = 1.0,
                                      enter_editmode: bool = False,
                                      align: typing.Union[int, str] = 'WORLD',
                                      location: float = (0.0, 0.0, 0.0),
                                      rotation: float = (0.0, 0.0, 0.0)):
    '''Construct a Nurbs surface Curve 

    :param radius: Radius 
    :type radius: float
    :param enter_editmode: Enter Editmode, Enter editmode when adding this object 
    :type enter_editmode: bool
    :param align: Align, The alignment of the new objectWORLD World, Align the new object to the world.VIEW View, Align the new object to the view.CURSOR 3D Cursor, Use the 3D cursor orientation for the new object. 
    :type align: typing.Union[int, str]
    :param location: Location, Location for the newly added object 
    :type location: float
    :param rotation: Rotation, Rotation for the newly added object 
    :type rotation: float
    '''

    pass


def primitive_nurbs_surface_cylinder_add(
        radius: float = 1.0,
        enter_editmode: bool = False,
        align: typing.Union[int, str] = 'WORLD',
        location: float = (0.0, 0.0, 0.0),
        rotation: float = (0.0, 0.0, 0.0)):
    '''Construct a Nurbs surface Cylinder 

    :param radius: Radius 
    :type radius: float
    :param enter_editmode: Enter Editmode, Enter editmode when adding this object 
    :type enter_editmode: bool
    :param align: Align, The alignment of the new objectWORLD World, Align the new object to the world.VIEW View, Align the new object to the view.CURSOR 3D Cursor, Use the 3D cursor orientation for the new object. 
    :type align: typing.Union[int, str]
    :param location: Location, Location for the newly added object 
    :type location: float
    :param rotation: Rotation, Rotation for the newly added object 
    :type rotation: float
    '''

    pass


def primitive_nurbs_surface_sphere_add(radius: float = 1.0,
                                       enter_editmode: bool = False,
                                       align: typing.Union[int, str] = 'WORLD',
                                       location: float = (0.0, 0.0, 0.0),
                                       rotation: float = (0.0, 0.0, 0.0)):
    '''Construct a Nurbs surface Sphere 

    :param radius: Radius 
    :type radius: float
    :param enter_editmode: Enter Editmode, Enter editmode when adding this object 
    :type enter_editmode: bool
    :param align: Align, The alignment of the new objectWORLD World, Align the new object to the world.VIEW View, Align the new object to the view.CURSOR 3D Cursor, Use the 3D cursor orientation for the new object. 
    :type align: typing.Union[int, str]
    :param location: Location, Location for the newly added object 
    :type location: float
    :param rotation: Rotation, Rotation for the newly added object 
    :type rotation: float
    '''

    pass


def primitive_nurbs_surface_surface_add(
        radius: float = 1.0,
        enter_editmode: bool = False,
        align: typing.Union[int, str] = 'WORLD',
        location: float = (0.0, 0.0, 0.0),
        rotation: float = (0.0, 0.0, 0.0)):
    '''Construct a Nurbs surface Patch 

    :param radius: Radius 
    :type radius: float
    :param enter_editmode: Enter Editmode, Enter editmode when adding this object 
    :type enter_editmode: bool
    :param align: Align, The alignment of the new objectWORLD World, Align the new object to the world.VIEW View, Align the new object to the view.CURSOR 3D Cursor, Use the 3D cursor orientation for the new object. 
    :type align: typing.Union[int, str]
    :param location: Location, Location for the newly added object 
    :type location: float
    :param rotation: Rotation, Rotation for the newly added object 
    :type rotation: float
    '''

    pass


def primitive_nurbs_surface_torus_add(radius: float = 1.0,
                                      enter_editmode: bool = False,
                                      align: typing.Union[int, str] = 'WORLD',
                                      location: float = (0.0, 0.0, 0.0),
                                      rotation: float = (0.0, 0.0, 0.0)):
    '''Construct a Nurbs surface Torus 

    :param radius: Radius 
    :type radius: float
    :param enter_editmode: Enter Editmode, Enter editmode when adding this object 
    :type enter_editmode: bool
    :param align: Align, The alignment of the new objectWORLD World, Align the new object to the world.VIEW View, Align the new object to the view.CURSOR 3D Cursor, Use the 3D cursor orientation for the new object. 
    :type align: typing.Union[int, str]
    :param location: Location, Location for the newly added object 
    :type location: float
    :param rotation: Rotation, Rotation for the newly added object 
    :type rotation: float
    '''

    pass



import sys
import typing
CLIPPING = None
'''constant value 2 '''

KERNING_DEFAULT = None
'''constant value 8 '''

MONOCHROME = None
'''constant value 128 '''

ROTATION = None
'''constant value 1 '''

SHADOW = None
'''constant value 4 '''

WORD_WRAP = None
'''constant value 64 '''


def aspect(fontid: int, aspect: float):
    '''Set the aspect for drawing text. 

    :param fontid: The id of the typeface as returned by blf.load(), for default font use 0. 
    :type fontid: int
    :param aspect: The aspect ratio for text drawing to use. 
    :type aspect: float
    '''

    pass


def clipping(fontid: int, xmin: float, ymin: float, xmax: float, ymax: float):
    '''Set the clipping, enable/disable using CLIPPING. 

    :param fontid: The id of the typeface as returned by blf.load(), for default font use 0. 
    :type fontid: int
    :param xmin: Clip the drawing area by these bounds. 
    :type xmin: float
    :param ymin: Clip the drawing area by these bounds. 
    :type ymin: float
    :param xmax: Clip the drawing area by these bounds. 
    :type xmax: float
    :param ymax: Clip the drawing area by these bounds. 
    :type ymax: float
    '''

    pass


def color(fontid: int, r: float, g: float, b: float, a: float):
    '''Set the color for drawing text. 

    :param fontid: The id of the typeface as returned by blf.load(), for default font use 0. 
    :type fontid: int
    :param r: red channel 0.0 - 1.0. 
    :type r: float
    :param g: green channel 0.0 - 1.0. 
    :type g: float
    :param b: blue channel 0.0 - 1.0. 
    :type b: float
    :param a: alpha channel 0.0 - 1.0. 
    :type a: float
    '''

    pass


def dimensions(fontid: int, text: str):
    '''Return the width and height of the text. 

    :param fontid: The id of the typeface as returned by blf.load(), for default font use 0. 
    :type fontid: int
    :param text: the text to draw. 
    :type text: str
    :return:  the width and height of the text. 
    '''

    pass


def disable(fontid: int, option: int):
    '''Disable option. 

    :param fontid: The id of the typeface as returned by blf.load(), for default font use 0. 
    :type fontid: int
    :param option: One of ROTATION, CLIPPING, SHADOW or KERNING_DEFAULT. 
    :type option: int
    '''

    pass


def draw(fontid: int, text: str):
    '''Draw text in the current context. 

    :param fontid: The id of the typeface as returned by blf.load(), for default font use 0. 
    :type fontid: int
    :param text: the text to draw. 
    :type text: str
    '''

    pass


def enable(fontid: int, option: int):
    '''Enable option. 

    :param fontid: The id of the typeface as returned by blf.load(), for default font use 0. 
    :type fontid: int
    :param option: One of ROTATION, CLIPPING, SHADOW or KERNING_DEFAULT. 
    :type option: int
    '''

    pass


def load(filename: str):
    '''Load a new font. 

    :param filename: the filename of the font. 
    :type filename: str
    :return:  the new font’s fontid or -1 if there was an error. 
    '''

    pass


def position(fontid: int, x: float, y: float, z: float):
    '''Set the position for drawing text. 

    :param fontid: The id of the typeface as returned by blf.load(), for default font use 0. 
    :type fontid: int
    :param x: X axis position to draw the text. 
    :type x: float
    :param y: Y axis position to draw the text. 
    :type y: float
    :param z: Z axis position to draw the text. 
    :type z: float
    '''

    pass


def rotation(fontid: int, angle: float):
    '''Set the text rotation angle, enable/disable using ROTATION. 

    :param fontid: The id of the typeface as returned by blf.load(), for default font use 0. 
    :type fontid: int
    :param angle: The angle for text drawing to use. 
    :type angle: float
    '''

    pass


def shadow(fontid: int, level: int, r: float, g: float, b: float, a: float):
    '''Shadow options, enable/disable using SHADOW . 

    :param fontid: The id of the typeface as returned by blf.load(), for default font use 0. 
    :type fontid: int
    :param level: The blur level, can be 3, 5 or 0. 
    :type level: int
    :param r: Shadow color (red channel 0.0 - 1.0). 
    :type r: float
    :param g: Shadow color (green channel 0.0 - 1.0). 
    :type g: float
    :param b: Shadow color (blue channel 0.0 - 1.0). 
    :type b: float
    :param a: Shadow color (alpha channel 0.0 - 1.0). 
    :type a: float
    '''

    pass


def shadow_offset(fontid: int, x: float, y: float):
    '''Set the offset for shadow text. 

    :param fontid: The id of the typeface as returned by blf.load(), for default font use 0. 
    :type fontid: int
    :param x: Vertical shadow offset value in pixels. 
    :type x: float
    :param y: Horizontal shadow offset value in pixels. 
    :type y: float
    '''

    pass


def size(fontid: int, size: int, dpi: int):
    '''Set the size and dpi for drawing text. 

    :param fontid: The id of the typeface as returned by blf.load(), for default font use 0. 
    :type fontid: int
    :param size: Point size of the font. 
    :type size: int
    :param dpi: dots per inch value to use for drawing. 
    :type dpi: int
    '''

    pass


def unload(filename: str):
    '''Unload an existing font. 

    :param filename: the filename of the font. 
    :type filename: str
    '''

    pass


def word_wrap(fontid: int, wrap_width: int):
    '''Set the wrap width, enable/disable using WORD_WRAP. 

    :param fontid: The id of the typeface as returned by blf.load(), for default font use 0. 
    :type fontid: int
    :param wrap_width: The width (in pixels) to wrap words at. 
    :type wrap_width: int
    '''

    pass



import sys
import typing


def change_frame(frame: float = 0.0, snap: bool = False):
    '''Interactively change the current frame number 

    :param frame: Frame 
    :type frame: float
    :param snap: Snap 
    :type snap: bool
    '''

    pass


def channel_select_keys(extend: bool = False):
    '''Select all keyframes of channel under mouse 

    :param extend: Extend, Extend selection 
    :type extend: bool
    '''

    pass


def channels_clean_empty():
    '''Delete all empty animation data containers from visible data-blocks 

    '''

    pass


def channels_click(extend: bool = False, children_only: bool = False):
    '''Handle mouse-clicks over animation channels 

    :param extend: Extend Select 
    :type extend: bool
    :param children_only: Select Children Only 
    :type children_only: bool
    '''

    pass


def channels_collapse(all: bool = True):
    '''Collapse (i.e. close) all selected expandable animation channels 

    :param all: All, Collapse all channels (not just selected ones) 
    :type all: bool
    '''

    pass


def channels_delete():
    '''Delete all selected animation channels 

    '''

    pass


def channels_editable_toggle(mode: typing.Union[int, str] = 'TOGGLE',
                             type: typing.Union[int, str] = 'PROTECT'):
    '''Toggle editability of selected channels 

    :param mode: Mode 
    :type mode: typing.Union[int, str]
    :param type: Type 
    :type type: typing.Union[int, str]
    '''

    pass


def channels_expand(all: bool = True):
    '''Expand (i.e. open) all selected expandable animation channels 

    :param all: All, Expand all channels (not just selected ones) 
    :type all: bool
    '''

    pass


def channels_fcurves_enable():
    '''Clears ‘disabled’ tag from all F-Curves to get broken F-Curves working again 

    '''

    pass


def channels_find(query: str = "Query"):
    '''Filter the set of channels shown to only include those with matching names 

    :param query: Text to search for in channel names 
    :type query: str
    '''

    pass


def channels_group(name: str = "NewGroup"):
    '''Add selected F-Curves to a new group 

    :param name: Name, Name of newly created group 
    :type name: str
    '''

    pass


def channels_move(direction: typing.Union[int, str] = 'DOWN'):
    '''Rearrange selected animation channels 

    :param direction: Direction 
    :type direction: typing.Union[int, str]
    '''

    pass


def channels_rename():
    '''Rename animation channel under mouse 

    '''

    pass


def channels_select_all(action: typing.Union[int, str] = 'TOGGLE'):
    '''Toggle selection of all animation channels 

    :param action: Action, Selection action to executeTOGGLE Toggle, Toggle selection for all elements.SELECT Select, Select all elements.DESELECT Deselect, Deselect all elements.INVERT Invert, Invert selection of all elements. 
    :type action: typing.Union[int, str]
    '''

    pass


def channels_select_box(xmin: int = 0,
                        xmax: int = 0,
                        ymin: int = 0,
                        ymax: int = 0,
                        wait_for_input: bool = True,
                        deselect: bool = False,
                        extend: bool = True):
    '''Select all animation channels within the specified region 

    :param xmin: X Min 
    :type xmin: int
    :param xmax: X Max 
    :type xmax: int
    :param ymin: Y Min 
    :type ymin: int
    :param ymax: Y Max 
    :type ymax: int
    :param wait_for_input: Wait for Input 
    :type wait_for_input: bool
    :param deselect: Deselect, Deselect rather than select items 
    :type deselect: bool
    :param extend: Extend, Extend selection instead of deselecting everything first 
    :type extend: bool
    '''

    pass


def channels_setting_disable(mode: typing.Union[int, str] = 'DISABLE',
                             type: typing.Union[int, str] = 'PROTECT'):
    '''Disable specified setting on all selected animation channels 

    :param mode: Mode 
    :type mode: typing.Union[int, str]
    :param type: Type 
    :type type: typing.Union[int, str]
    '''

    pass


def channels_setting_enable(mode: typing.Union[int, str] = 'ENABLE',
                            type: typing.Union[int, str] = 'PROTECT'):
    '''Enable specified setting on all selected animation channels 

    :param mode: Mode 
    :type mode: typing.Union[int, str]
    :param type: Type 
    :type type: typing.Union[int, str]
    '''

    pass


def channels_setting_toggle(mode: typing.Union[int, str] = 'TOGGLE',
                            type: typing.Union[int, str] = 'PROTECT'):
    '''Toggle specified setting on all selected animation channels 

    :param mode: Mode 
    :type mode: typing.Union[int, str]
    :param type: Type 
    :type type: typing.Union[int, str]
    '''

    pass


def channels_ungroup():
    '''Remove selected F-Curves from their current groups 

    '''

    pass


def clear_useless_actions(only_unused: bool = True):
    '''Mark actions with no F-Curves for deletion after save & reload of file preserving “action libraries” 

    :param only_unused: Only Unused, Only unused (Fake User only) actions get considered 
    :type only_unused: bool
    '''

    pass


def copy_driver_button():
    '''Copy the driver for the highlighted button 

    '''

    pass


def driver_button_add():
    '''Add driver for the property under the cursor 

    '''

    pass


def driver_button_edit():
    '''Edit the drivers for the property connected represented by the highlighted button 

    '''

    pass


def driver_button_remove(all: bool = True):
    '''Remove the driver(s) for the property(s) connected represented by the highlighted button 

    :param all: All, Delete drivers for all elements of the array 
    :type all: bool
    '''

    pass


def end_frame_set():
    '''Set the current frame as the preview or scene end frame 

    '''

    pass


def keyframe_clear_button(all: bool = True):
    '''Clear all keyframes on the currently active property 

    :param all: All, Clear keyframes from all elements of the array 
    :type all: bool
    '''

    pass


def keyframe_clear_v3d():
    '''Remove all keyframe animation for selected objects 

    '''

    pass


def keyframe_delete(type: typing.Union[int, str] = 'DEFAULT',
                    confirm_success: bool = True):
    '''Delete keyframes on the current frame for all properties in the specified Keying Set 

    :param type: Keying Set, The Keying Set to use 
    :type type: typing.Union[int, str]
    :param confirm_success: Confirm Successful Delete, Show a popup when the keyframes get successfully removed 
    :type confirm_success: bool
    '''

    pass


def keyframe_delete_button(all: bool = True):
    '''Delete current keyframe of current UI-active property 

    :param all: All, Delete keyframes from all elements of the array 
    :type all: bool
    '''

    pass


def keyframe_delete_by_name(type="Type", confirm_success: bool = True):
    '''Alternate access to ‘Delete Keyframe’ for keymaps to use 

    :param confirm_success: Confirm Successful Delete, Show a popup when the keyframes get successfully removed 
    :type confirm_success: bool
    '''

    pass


def keyframe_delete_v3d():
    '''Remove keyframes on current frame for selected objects and bones 

    '''

    pass


def keyframe_insert(type: typing.Union[int, str] = 'DEFAULT',
                    confirm_success: bool = True):
    '''Insert keyframes on the current frame for all properties in the specified Keying Set 

    :param type: Keying Set, The Keying Set to use 
    :type type: typing.Union[int, str]
    :param confirm_success: Confirm Successful Insert, Show a popup when the keyframes get successfully added 
    :type confirm_success: bool
    '''

    pass


def keyframe_insert_button(all: bool = True):
    '''Insert a keyframe for current UI-active property 

    :param all: All, Insert a keyframe for all element of the array 
    :type all: bool
    '''

    pass


def keyframe_insert_by_name(type="Type", confirm_success: bool = True):
    '''Alternate access to ‘Insert Keyframe’ for keymaps to use 

    :param confirm_success: Confirm Successful Insert, Show a popup when the keyframes get successfully added 
    :type confirm_success: bool
    '''

    pass


def keyframe_insert_menu(type: typing.Union[int, str] = 'DEFAULT',
                         confirm_success: bool = False,
                         always_prompt: bool = False):
    '''Insert Keyframes for specified Keying Set, with menu of available Keying Sets if undefined 

    :param type: Keying Set, The Keying Set to use 
    :type type: typing.Union[int, str]
    :param confirm_success: Confirm Successful Insert, Show a popup when the keyframes get successfully added 
    :type confirm_success: bool
    :param always_prompt: Always Show Menu 
    :type always_prompt: bool
    '''

    pass


def keying_set_active_set(type: typing.Union[int, str] = 'DEFAULT'):
    '''Select a new keying set as the active one 

    :param type: Keying Set, The Keying Set to use 
    :type type: typing.Union[int, str]
    '''

    pass


def keying_set_add():
    '''Add a new (empty) Keying Set to the active Scene 

    '''

    pass


def keying_set_export(filepath: str = "",
                      filter_folder: bool = True,
                      filter_text: bool = True,
                      filter_python: bool = True):
    '''Export Keying Set to a python script 

    :param filepath: filepath 
    :type filepath: str
    :param filter_folder: Filter folders 
    :type filter_folder: bool
    :param filter_text: Filter text 
    :type filter_text: bool
    :param filter_python: Filter python 
    :type filter_python: bool
    '''

    pass


def keying_set_path_add():
    '''Add empty path to active Keying Set 

    '''

    pass


def keying_set_path_remove():
    '''Remove active Path from active Keying Set 

    '''

    pass


def keying_set_remove():
    '''Remove the active Keying Set 

    '''

    pass


def keyingset_button_add(all: bool = True):
    '''Add current UI-active property to current keying set 

    :param all: All, Add all elements of the array to a Keying Set 
    :type all: bool
    '''

    pass


def keyingset_button_remove():
    '''Remove current UI-active property from current keying set 

    '''

    pass


def paste_driver_button():
    '''Paste the driver in the copy/paste buffer for the highlighted button 

    '''

    pass


def previewrange_clear():
    '''Clear Preview Range 

    '''

    pass


def previewrange_set(xmin: int = 0,
                     xmax: int = 0,
                     ymin: int = 0,
                     ymax: int = 0,
                     wait_for_input: bool = True):
    '''Interactively define frame range used for playback 

    :param xmin: X Min 
    :type xmin: int
    :param xmax: X Max 
    :type xmax: int
    :param ymin: Y Min 
    :type ymin: int
    :param ymax: Y Max 
    :type ymax: int
    :param wait_for_input: Wait for Input 
    :type wait_for_input: bool
    '''

    pass


def start_frame_set():
    '''Set the current frame as the preview or scene start frame 

    '''

    pass


def update_animated_transform_constraints(use_convert_to_radians: bool = True):
    '''Update fcurves/drivers affecting Transform constraints (use it with files from 2.70 and earlier) 

    :param use_convert_to_radians: Convert To Radians, Convert fcurves/drivers affecting rotations to radians (Warning: use this only once!) 
    :type use_convert_to_radians: bool
    '''

    pass
